# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.jwt._jwk_set_converter."""

from absl.testing import absltest
from absl.testing import parameterized

from tink.proto import tink_pb2
import tink
from tink import jwt
from tink import secret_key_access

ES256_KEYSET = (
    '{"primaryKeyId":282600252,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPublicKey",'
    '"value":"EAEaIBDPI66hjLHvjxmUJ2nyHIBDmdOtQ4gPsvWgYYgZ0gygIiBTEK0rTACpAb97m'
    '+mvtJKAk0q3mHjPcUZm0C4EueDW4Q==",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":282600252,"outputPrefixType":"RAW"}]}')

ES256_JWK_SET = ('{"keys":[{'
                 '"kty":"EC",'
                 '"crv":"P-256",'
                 '"x":"EM8jrqGMse-PGZQnafIcgEOZ061DiA-y9aBhiBnSDKA",'
                 '"y":"UxCtK0wAqQG_e5vpr7SSgJNKt5h4z3FGZtAuBLng1uE",'
                 '"use":"sig","alg":"ES256","key_ops":["verify"]}]}')

ES256_KEYSET_TINK = (
    '{"primaryKeyId":282600252,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPublicKey",'
    '"value":"EAEaIBDPI66hjLHvjxmUJ2nyHIBDmdOtQ4gPsvWgYYgZ0gygIiBTEK0rTACpAb97m'
    '+mvtJKAk0q3mHjPcUZm0C4EueDW4Q==",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":282600252,"outputPrefixType":"TINK"}]}')

ES256_JWK_SET_KID = ('{"keys":[{'
                     '"kty":"EC",'
                     '"crv":"P-256",'
                     '"x":"EM8jrqGMse-PGZQnafIcgEOZ061DiA-y9aBhiBnSDKA",'
                     '"y":"UxCtK0wAqQG_e5vpr7SSgJNKt5h4z3FGZtAuBLng1uE",'
                     '"use":"sig","alg":"ES256","key_ops":["verify"],'
                     '"kid":"ENgjPA"}]}')

ES384_KEYSET = (
    '{"primaryKeyId":456087424,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPublicKey",'
    '"value":"EAIaMQDSjvWihoKGmr4nlDuI/KkvuPvEZr+B4bU0MuXQQXgyNMGApFm2iTeotv7LC'
    'SsG3mQiMEHIMGx4wa+Y8yeJQWMiSpukpPM7jP9GqaykZQQ2GY/NLg/n9+BJtntgvFhG5gWLTg='
    '=","keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":456087424,"outputPrefixType":"RAW"}]}')

ES384_JWK_SET = (
    '{"keys":[{"kty":"EC","crv":"P-384",'
    '"x":"ANKO9aKGgoaavieUO4j8qS-4-8Rmv4HhtTQy5dBBeDI0wYCkWbaJN6i2_ssJKwbeZA",'
    '"y":"QcgwbHjBr5jzJ4lBYyJKm6Sk8zuM_0aprKRlBDYZj80uD-f34Em2e2C8WEbmBYtO",'
    '"use":"sig","alg":"ES384","key_ops":["verify"]}]}')

ES512_KEYSET = (
    '{"primaryKeyId":1570200439,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPublicKey",'
    '"value":"EAMaQgEV3nweRej6Z1/aPTqCkc1tQla5eVI68+qfwR1kB/wXCuYCB5otarhomUt64'
    'Fah/8Tjf0WJHMZyFr86RUitiRQm1SJCATht/NOX8RcbaEr1MaH+0BFTaepvpTzSfQ04C2P8VCo'
    'URB3GeVKk4VQh8O/KLSYfX+58bqEnaZ0G7W9qjHa2ols2",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":1570200439,"outputPrefixType":"RAW"}]}')

ES512_JWK_SET = (
    '{"keys":[{"kty":"EC","crv":"P-521",'
    '"x":"ARXefB5F6PpnX9o9OoKRzW1CVrl5Ujrz6p_BHWQH_BcK5gIHmi1quGiZS3rgVqH_xON_R'
    'YkcxnIWvzpFSK2JFCbV",'
    '"y":"ATht_NOX8RcbaEr1MaH-0BFTaepvpTzSfQ04C2P8VCoURB3GeVKk4VQh8O_KLSYfX-58b'
    'qEnaZ0G7W9qjHa2ols2",'
    '"use":"sig","alg":"ES512","key_ops":["verify"]}]}')

RS256_KEYSET = (
    '{"primaryKeyId":482168993,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPkcs1PublicKey"'
    ',"value":"EAEagQIAkspk37lGBqXmPPq2CL5KdDeRx7xFiTadpL3jc4nXaqftCtpM6qExfrc2'
    'JLaIsnwpwfGMClfe/alIs2GrT9fpM8oDeCccvC39DzZhsSFnAELggi3hnWNKRLfSV0UJzBI+5h'
    'Z6ifUsv8W8mSHKlsVMmvOfC2P5+l72qTwN6Le3hy6CxFp5s9pw011B7J3PU65sty6GI9sehB2B'
    '/n7nfiWw9YN5++pfwyoitzoMoVKOOpj7fFq88f8ArpC7kR1SBTe20Bt1AmpZDT2Dmfmlb/Q1UF'
    'jj/F3C77NCNQ344ZcAEI42HY+uighy5GdKQRHMoTT1OzyDG90ABjggQqDGW+zXzyIDAQAB",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":482168993,"outputPrefixType":"RAW"}]}')

RS256_JWK_SET = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AJLKZN-5Rgal5jz6tgi-SnQ3kce8RYk2naS943OJ12qn7QraTOqhMX63NiS2iLJ8KcHxj'
    'ApX3v2pSLNhq0_X6TPKA3gnHLwt_Q82YbEhZwBC4IIt4Z1jSkS30ldFCcwSPuYWeon1LL_FvJk'
    'hypbFTJrznwtj-fpe9qk8Dei3t4cugsRaebPacNNdQeydz1OubLcuhiPbHoQdgf5-534lsPWDe'
    'fvqX8MqIrc6DKFSjjqY-3xavPH_AK6Qu5EdUgU3ttAbdQJqWQ09g5n5pW_0NVBY4_xdwu-zQjU'
    'N-OGXABCONh2ProoIcuRnSkERzKE09Ts8gxvdAAY4IEKgxlvs188",'
    '"e":"AQAB","use":"sig","alg":"RS256","key_ops":["verify"]}]}')

RS256_KEYSET_TINK = (
    '{"primaryKeyId":482168993,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPkcs1PublicKey"'
    ',"value":"EAEagQIAkspk37lGBqXmPPq2CL5KdDeRx7xFiTadpL3jc4nXaqftCtpM6qExfrc2'
    'JLaIsnwpwfGMClfe/alIs2GrT9fpM8oDeCccvC39DzZhsSFnAELggi3hnWNKRLfSV0UJzBI+5h'
    'Z6ifUsv8W8mSHKlsVMmvOfC2P5+l72qTwN6Le3hy6CxFp5s9pw011B7J3PU65sty6GI9sehB2B'
    '/n7nfiWw9YN5++pfwyoitzoMoVKOOpj7fFq88f8ArpC7kR1SBTe20Bt1AmpZDT2Dmfmlb/Q1UF'
    'jj/F3C77NCNQ344ZcAEI42HY+uighy5GdKQRHMoTT1OzyDG90ABjggQqDGW+zXzyIDAQAB",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":482168993,"outputPrefixType":"TINK"}]}')

RS256_JWK_SET_KID = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AJLKZN-5Rgal5jz6tgi-SnQ3kce8RYk2naS943OJ12qn7QraTOqhMX63NiS2iLJ8KcHxj'
    'ApX3v2pSLNhq0_X6TPKA3gnHLwt_Q82YbEhZwBC4IIt4Z1jSkS30ldFCcwSPuYWeon1LL_FvJk'
    'hypbFTJrznwtj-fpe9qk8Dei3t4cugsRaebPacNNdQeydz1OubLcuhiPbHoQdgf5-534lsPWDe'
    'fvqX8MqIrc6DKFSjjqY-3xavPH_AK6Qu5EdUgU3ttAbdQJqWQ09g5n5pW_0NVBY4_xdwu-zQjU'
    'N-OGXABCONh2ProoIcuRnSkERzKE09Ts8gxvdAAY4IEKgxlvs188",'
    '"e":"AQAB","use":"sig","alg":"RS256","key_ops":["verify"],'
    '"kid":"HL1QoQ"}]}')

RS384_KEYSET = (
    '{"primaryKeyId":333504275,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPkcs1PublicKey"'
    ',"value":"EAIagQMAnlBY5WD7gVQjNKvrS2whLKzt0Eql72B6haZ17eKifNn4S49eGdBy9RLj'
    '/mvHXAbacrngt9fzi0iv/WQ57jUmtO1b/wLt5LYk9APsBYjywDCIe+u9UouikP7c3SBqjjQijZ'
    '50jgYbMY6cL7s2Gx5lI1vlGX3ZExLVYbNoI9VBFAWjSDefd6GugESxXQFnnO3p2GHOKryZLeDH'
    '/KzVacTq2/pVXKVH/9/EQzcLB0oYUljZ4vYQ4HCAcwnUZbirsRwA0350Dz0Mlj+3+9sSAF8FPA'
    '+F/wlIBkPqjJ26b80V5FU4mBTzvYoXGTjkD7+bxH9p28huJSU96P4WdG5PYVwI1VEYwGipkUIp'
    'MWjJ7dXAtmltHzM9vkUt2bsBe9vyJjmRXyoC6mHSJbSyOm9Dd8BENobcUL9h+aBoxruY+mU49k'
    'AHzzeAntn8C+vIrxN+X6N2EU9N8t9BF+mwYiBEsY54wx99RbRrY9yICfPBmQJGwXSxNCXBRrbJ'
    'yxkIVuqvACP5IgMBAAE=",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":333504275,"outputPrefixType":"RAW"}]}')

RS384_JWK_SET = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AJ5QWOVg-4FUIzSr60tsISys7dBKpe9geoWmde3ionzZ-EuPXhnQcvUS4_5rx1wG2nK54'
    'LfX84tIr_1kOe41JrTtW_8C7eS2JPQD7AWI8sAwiHvrvVKLopD-3N0gao40Io2edI4GGzGOnC-'
    '7NhseZSNb5Rl92RMS1WGzaCPVQRQFo0g3n3ehroBEsV0BZ5zt6dhhziq8mS3gx_ys1WnE6tv6V'
    'VylR__fxEM3CwdKGFJY2eL2EOBwgHMJ1GW4q7EcANN-dA89DJY_t_vbEgBfBTwPhf8JSAZD6oy'
    'dum_NFeRVOJgU872KFxk45A-_m8R_advIbiUlPej-FnRuT2FcCNVRGMBoqZFCKTFoye3VwLZpb'
    'R8zPb5FLdm7AXvb8iY5kV8qAuph0iW0sjpvQ3fARDaG3FC_YfmgaMa7mPplOPZAB883gJ7Z_Av'
    'ryK8Tfl-jdhFPTfLfQRfpsGIgRLGOeMMffUW0a2PciAnzwZkCRsF0sTQlwUa2ycsZCFbq'
    'rwAj-Q",'
    '"e":"AQAB","use":"sig","alg":"RS384","key_ops":["verify"]}]}')

RS512_KEYSET = (
    '{"primaryKeyId":705596479,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPkcs1PublicKey"'
    ',"value":"EAMagQQAkKxZ9IRzF56gh47RXLJzQ6lffcnBmQSwvxUDJ0wHpKZzfAawOn1uidbg'
    'EoQ3XWOgtNvi7QeKLE4GjQa5bY0xdRnu8nKjFcsvH+eu1sV8oVoZ984J5mT1mhwU6nt26p4xKy'
    'eapMhzYYNvKudQjQJ8SbpVOFpEiJ7j0ECMUd4Q8mCUqWsrXYE8+1CcHjprsIxdot+haCARc72R'
    'Bj9cLuBIhJNzlFXNmsYh8yoSiEYr/auRvg/kIlNlnlOK/rJM/jMXbB6FuWdePrtqZ+ce2TVyAR'
    'qjZJ0G0vZcPuvOhgS4LM7/Aeal84ZhIcHladSo/g8pK1eUhnRqRXJpsltwux+1XVJeg2a0FQ0B'
    'N3Ft25uu5jhfvGWXeTkQOR7LbpbxKTI+vumSy9dmY4UrgAG37N8Xj5/NeqBT51L3qE6tk2ZLoO'
    '7yjRjhADK5lnbb4iYWWvWd3kqyv0JVlxfDzjAaYtiduEUIdCe45MGk8DpCn9Lnjlunhm4QyQuf'
    'K8k8UPiBbWNEODI8pjTSEjs0wyMqhegBKAvtVEhr029bg3Lv7YjN9FDvx4usuWGc16bXkTqNgC'
    'K4KzPG7PwV120r6IVGflfpSkd5rrkzDY01fsP0mW57QCHA67bxqLUECr2dAfNzz6ddS9pqXQyX'
    'ZWCyWKcvTFsGrr1oECwDOmW+nUIHGklr9Q0iAwEAAQ==",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":705596479,"outputPrefixType":"RAW"}]}')

RS512_JWK_SET = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AJCsWfSEcxeeoIeO0Vyyc0OpX33JwZkEsL8VAydMB6Smc3wGsDp9bonW4BKEN11joLTb4'
    'u0HiixOBo0GuW2NMXUZ7vJyoxXLLx_nrtbFfKFaGffOCeZk9ZocFOp7duqeMSsnmqTIc2GDbyr'
    'nUI0CfEm6VThaRIie49BAjFHeEPJglKlrK12BPPtQnB46a7CMXaLfoWggEXO9kQY_XC7gSISTc'
    '5RVzZrGIfMqEohGK_2rkb4P5CJTZZ5Tiv6yTP4zF2wehblnXj67amfnHtk1cgEao2SdBtL2XD7'
    'rzoYEuCzO_wHmpfOGYSHB5WnUqP4PKStXlIZ0akVyabJbcLsftV1SXoNmtBUNATdxbdubruY4X'
    '7xll3k5EDkey26W8SkyPr7pksvXZmOFK4ABt-zfF4-fzXqgU-dS96hOrZNmS6Du8o0Y4QAyuZZ'
    '22-ImFlr1nd5Ksr9CVZcXw84wGmLYnbhFCHQnuOTBpPA6Qp_S545bp4ZuEMkLnyvJPFD4gW1jR'
    'DgyPKY00hI7NMMjKoXoASgL7VRIa9NvW4Ny7-2IzfRQ78eLrLlhnNem15E6jYAiuCszxuz8Fdd'
    'tK-iFRn5X6UpHea65Mw2NNX7D9Jlue0AhwOu28ai1BAq9nQHzc8-nXUvaal0Ml2VgslinL0xbB'
    'q69aBAsAzplvp1CBxpJa_UN",'
    '"e":"AQAB","use":"sig","alg":"RS512","key_ops":["verify"]}]}')

PRIVATEKEY_KEYSET = (
    '{"primaryKeyId":152493399,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPrivateKey",'
    '"value":"EkYQARogaHkaakArEB51RyZ236S5x3BxaNTFycWuXIGZF8adZ2UiIFlZT7MFogZ'
    '8ARbS1URIAPcpw8A0g2uwAHRkBqGUiCU2GiBI4jtU/59Zajohgeezi2BXB13O8IJh8V3b0it'
    'q5zyy5Q==",'
    '"keyMaterialType":"ASYMMETRIC_PRIVATE"'
    '},"status":"ENABLED","keyId":152493399,"outputPrefixType":"RAW"}]}')

KEYSET_WITH_DISABLED_KEY = (
    '{"primaryKeyId":282600252,"key":['
    '{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPublicKey",'
    '"value":"EAEaIBDPI66hjLHvjxmUJ2nyHIBDmdOtQ4gPsvWgYYgZ0gygIiBTEK0rTACpAb97m'
    '+mvtJKAk0'
    'q3mHjPcUZm0C4EueDW4Q==",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":282600252,"outputPrefixType":"RAW"},'
    '{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPkcs1PublicKey"'
    ','
    '"value":"EAEagQIAkspk37lGBqXmPPq2CL5KdDeRx7xFiTadpL3jc4nXaqftCtpM6qExfrc2J'
    'LaIsnwpwfGMClfe/alIs2GrT9fpM8oDeCccvC39DzZhsSFnAELggi3hnWNKRLfSV0UJzBI+5hZ'
    '6ifUsv8W8mSHKlsVMmvOfC2P5+l72qTwN6Le3hy6CxFp5s9pw011B7J3PU65sty6GI9sehB2B/'
    'n7nfiWw9YN5++pfwyoitzoMoVKOOpj7fFq88f8ArpC7kR1SBTe20Bt1AmpZDT2Dmfmlb/Q1UFj'
    'j/F3C77NCNQ344ZcAEI42HY+uighy5GdKQRHMoTT1OzyDG90ABjggQqDGW+zXzyIDAQAB",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"DISABLED","keyId":482168993,"outputPrefixType":"RAW"}]}')

PS256_KEYSET = (
    '{"primaryKeyId":1508587714,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPssPublicKey",'
    '"value":"EAEagQMAj7Eud2n5G11qsdtjpgGWjW4cAKalSE1atm7d+Cp8biRX9wbmLJRMUvoO2'
    'j7Sp9Szx1TMmksY2Ugf/7+Nv9fY7vBbmxOiBQVTvikWn0FgPwhFTXTz+9fhGjM6E6sdSOUzjM6'
    'nsPulKqOQ8Aed+TLIlgvwuSTF4B5d6QkZWBymq7My6vV+epzWnoLpVDzCHh+c35r81Pyrj6tiT'
    'PQzPLN2ixeanclMjx8deNwlak3vwBdMDgwQ63rVCo2eWDS/BYK4rG22luSTDVfQVHU1NXlwXEn'
    'b/eONFSF6ZbD6JXFMT3uHT4okTOrX4Kd34stbPIUtZFUy3XiSeCGtghBXLMf/ge113Q9WDJ+RN'
    '1Xa4vgHJCO0+VO+cAugVkiu9UgsPP8o/r7tA2aP/Ps8EHYa1IaZg75vnrMZPvsTH7WG2SjSgW9'
    'GLLsbNJLFFqLFMwPuZPe8BbgvimPdStXasX/PN6DLKoK2PaT0I+iLK9mRi1Z4OjFbl9KAZXXEl'
    'hAQTzrEI2adIgMBAAE=",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":1508587714,"outputPrefixType":"RAW"}]}')

PS256_JWK_SET = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AI-xLndp-RtdarHbY6YBlo1uHACmpUhNWrZu3fgqfG4kV_cG5iyUTFL6Dto-0qfUs8dUz'
    'JpLGNlIH_-_jb_X2O7wW5sTogUFU74pFp9BYD8IRU108_vX4RozOhOrHUjlM4zOp7D7pSqjkPA'
    'HnfkyyJYL8LkkxeAeXekJGVgcpquzMur1fnqc1p6C6VQ8wh4fnN-a_NT8q4-rYkz0MzyzdosXm'
    'p3JTI8fHXjcJWpN78AXTA4MEOt61QqNnlg0vwWCuKxttpbkkw1X0FR1NTV5cFxJ2_3jjRUhemW'
    'w-iVxTE97h0-KJEzq1-Cnd-LLWzyFLWRVMt14knghrYIQVyzH_4Htdd0PVgyfkTdV2uL4ByQjt'
    'PlTvnALoFZIrvVILDz_KP6-7QNmj_z7PBB2GtSGmYO-b56zGT77Ex-1htko0oFvRiy7GzSSxRa'
    'ixTMD7mT3vAW4L4pj3UrV2rF_zzegyyqCtj2k9CPoiyvZkYtWeDoxW5fSgGV1xJYQEE86xCNmn'
    'Q",'
    '"e":"AQAB","use":"sig","alg":"PS256","key_ops":["verify"]}]}')

PS256_KEYSET_TINK = (
    '{"primaryKeyId":1508587714,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPssPublicKey",'
    '"value":"EAEagQMAj7Eud2n5G11qsdtjpgGWjW4cAKalSE1atm7d+Cp8biRX9wbmLJRMUvoO2'
    'j7Sp9Szx1TMmksY2Ugf/7+Nv9fY7vBbmxOiBQVTvikWn0FgPwhFTXTz+9fhGjM6E6sdSOUzjM6'
    'nsPulKqOQ8Aed+TLIlgvwuSTF4B5d6QkZWBymq7My6vV+epzWnoLpVDzCHh+c35r81Pyrj6tiT'
    'PQzPLN2ixeanclMjx8deNwlak3vwBdMDgwQ63rVCo2eWDS/BYK4rG22luSTDVfQVHU1NXlwXEn'
    'b/eONFSF6ZbD6JXFMT3uHT4okTOrX4Kd34stbPIUtZFUy3XiSeCGtghBXLMf/ge113Q9WDJ+RN'
    '1Xa4vgHJCO0+VO+cAugVkiu9UgsPP8o/r7tA2aP/Ps8EHYa1IaZg75vnrMZPvsTH7WG2SjSgW9'
    'GLLsbNJLFFqLFMwPuZPe8BbgvimPdStXasX/PN6DLKoK2PaT0I+iLK9mRi1Z4OjFbl9KAZXXEl'
    'hAQTzrEI2adIgMBAAE=",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":1508587714,"outputPrefixType":"TINK"}]}')

PS256_JWK_SET_KID = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AI-xLndp-RtdarHbY6YBlo1uHACmpUhNWrZu3fgqfG4kV_cG5iyUTFL6Dto-0qfUs8dUz'
    'JpLGNlIH_-_jb_X2O7wW5sTogUFU74pFp9BYD8IRU108_vX4RozOhOrHUjlM4zOp7D7pSqjkPA'
    'HnfkyyJYL8LkkxeAeXekJGVgcpquzMur1fnqc1p6C6VQ8wh4fnN-a_NT8q4-rYkz0MzyzdosXm'
    'p3JTI8fHXjcJWpN78AXTA4MEOt61QqNnlg0vwWCuKxttpbkkw1X0FR1NTV5cFxJ2_3jjRUhemW'
    'w-iVxTE97h0-KJEzq1-Cnd-LLWzyFLWRVMt14knghrYIQVyzH_4Htdd0PVgyfkTdV2uL4ByQjt'
    'PlTvnALoFZIrvVILDz_KP6-7QNmj_z7PBB2GtSGmYO-b56zGT77Ex-1htko0oFvRiy7GzSSxRa'
    'ixTMD7mT3vAW4L4pj3UrV2rF_zzegyyqCtj2k9CPoiyvZkYtWeDoxW5fSgGV1xJYQEE86xCNmn'
    'Q",'
    '"e":"AQAB","use":"sig","alg":"PS256","key_ops":["verify"],'
    '"kid":"Wes4wg"}]}')

PS384_KEYSET = (
    '{"primaryKeyId":1042230435,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPssPublicKey",'
    '"value":"EAIagQMAv6a0OergWYmY1k6l6vx6Of5+RxCeeQ9jMTXQyvO0GCgMDExxtqVS8S25e'
    'hZ5LNDIiGjhE3v2++D7QEjnzOC5UqI1ZwPxUBSrOaf5oDbJ9vBc2c7wDyJhRV8UobQSpzunD4k'
    'XypVhytjwRdiP61vG0C/eL0x+LijtM/XVee1Y+5mWrypVrB6EHKtdkMx2WIYNpsFOForFrr6Jz'
    'LbWfDRWoqbCXKYivnw+CSE38ddW1XsrAT76E2Vf+womuwyBbkjLaiWvNxNFBTap2IaBLKAni6x'
    '7pqYCeu1n9eMUi41oz9QM8xfOvpH+wubc2PjwyTsb1FDTLnhV36tQLTVGdQdCDMF2Z8Agrnio3'
    'n1SFjSbYgFyVtpCwFKM2Z0zfO7k9jVbYYkzglzkJfp/lQrsuWqe4CVJjFE1H4BxcU7L0j8755k'
    'GJI08h1b7LPgqJcPgtHjcqbxHFU2yOf7mNGlW7YTnoQBO0StzQUk7kEw3X0+niEwX/L8j'
    'qW4YMbxrGdAfkTnPIgMBAAE=",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":1042230435,"outputPrefixType":"RAW"}]}')

PS384_JWK_SET = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AL-mtDnq4FmJmNZOper8ejn-fkcQnnkPYzE10MrztBgoDAxMcbalUvEtuXoWeSzQyIho4'
    'RN79vvg-0BI58zguVKiNWcD8VAUqzmn-aA2yfbwXNnO8A8iYUVfFKG0Eqc7pw-JF8qVYcrY8EX'
    'Yj-tbxtAv3i9Mfi4o7TP11XntWPuZlq8qVawehByrXZDMdliGDabBThaKxa6-icy21nw0VqKmw'
    'lymIr58PgkhN_HXVtV7KwE--hNlX_sKJrsMgW5Iy2olrzcTRQU2qdiGgSygJ4use6amAnrtZ_X'
    'jFIuNaM_UDPMXzr6R_sLm3Nj48Mk7G9RQ0y54Vd-rUC01RnUHQgzBdmfAIK54qN59UhY0m2IBc'
    'lbaQsBSjNmdM3zu5PY1W2GJM4Jc5CX6f5UK7LlqnuAlSYxRNR-AcXFOy9I_O-eZBiSNPIdW-yz'
    '4KiXD4LR43Km8RxVNsjn-5jRpVu2E56EATtErc0FJO5BMN19Pp4hMF_y_I6luGDG8axnQH5E5z'
    'w",'
    '"e":"AQAB","use":"sig","alg":"PS384","key_ops":["verify"]}]}')

PS512_KEYSET = (
    '{"primaryKeyId":257081135,"key":[{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPssPublicKey",'
    '"value":"EAMagQQAnOUQvBwNRgeI3zlzIhVo4NzFVCsQn9hd2EIclz6cWBRMFr4EX5lXLK0St'
    'SIB7EQP4ciHa+vr59sOgMFMC2kiXRUXNtl99QhGwH0YjbWeDC50PKEAjH1hhhPgSw2dFcUVs4j'
    'bScDrwNn1sQ8rkgSNczvQNpV1MtBhS/CC1PxVF88JaejG2zr+unoFlw7xnqxBWMzNrMHZHwqga'
    '2vL3inSbvA/RGQjnE2DzQSwZkXthGSwYBjOYbGawMN4onkAx/myHMyTg/TLAqG9GUyB0DVelvV'
    'oGZG/QJBY2Fp2FlpOQRKeBr6pC7Lk8zZL4GJk264KoOpG8v1t7PveN+STIdTE2D548K+GDOvsv'
    'rO4ZhofS/iqN9xLucuU1HkqKUqyLvMxsWum8Zhp7zinFdBnDOgeheOHUgN/iwjupk6u1Svt+RW'
    'NJsfb2l0jrvzf0cRMbPeLZRmpDwBxBvXWo61u6uaBEVb+ooZ6K5+hx3Rld7wXktjYIZzHqUr39'
    'P5yTw28b8Y2dPFWR4vwr2/0zBxcDmTRRtQ7vPOtZPD0/LVIXkgbBiLILpycnucWt9Lq9Hc62KF'
    'iTQOAuuOxz7ObBegXjnFupiZZ9PyzO5WgT9lRpH7U7tzGLAjV+AUpjH6HA1o6bRLKOHFBPS+I9'
    'IqAYb/RpF6M/6hCmC2Rz64yYzR3y4vHKGMiAwEAAQ==",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":257081135,"outputPrefixType":"RAW"}]}')

PS512_JWK_SET = (
    '{"keys":[{"kty":"RSA",'
    '"n":"AJzlELwcDUYHiN85cyIVaODcxVQrEJ_YXdhCHJc-nFgUTBa-BF-ZVyytErUiAexED-HIh'
    '2vr6-fbDoDBTAtpIl0VFzbZffUIRsB9GI21ngwudDyhAIx9YYYT4EsNnRXFFbOI20nA68DZ9bE'
    'PK5IEjXM70DaVdTLQYUvwgtT8VRfPCWnoxts6_rp6BZcO8Z6sQVjMzazB2R8KoGtry94p0m7wP'
    '0RkI5xNg80EsGZF7YRksGAYzmGxmsDDeKJ5AMf5shzMk4P0ywKhvRlMgdA1Xpb1aBmRv0CQWNh'
    'adhZaTkESnga-qQuy5PM2S-BiZNuuCqDqRvL9bez73jfkkyHUxNg-ePCvhgzr7L6zuGYaH0v4q'
    'jfcS7nLlNR5KilKsi7zMbFrpvGYae84pxXQZwzoHoXjh1IDf4sI7qZOrtUr7fkVjSbH29pdI67'
    '839HETGz3i2UZqQ8AcQb11qOtburmgRFW_qKGeiufocd0ZXe8F5LY2CGcx6lK9_T-ck8NvG_GN'
    'nTxVkeL8K9v9MwcXA5k0UbUO7zzrWTw9Py1SF5IGwYiyC6cnJ7nFrfS6vR3OtihYk0DgLrjsc-'
    'zmwXoF45xbqYmWfT8szuVoE_ZUaR-1O7cxiwI1fgFKYx-hwNaOm0SyjhxQT0viPSKgGG_0aRej'
    'P-oQpgtkc-uMmM0d8uLxyhj",'
    '"e":"AQAB","use":"sig","alg":"PS512","key_ops":["verify"]}]}')

HS256_KEYSET = """
    {
      "primaryKeyId": 872908418,
      "key": [
        {
          "keyData": {
            "typeUrl": "type.googleapis.com/google.crypto.tink.JwtHmacKey",
            "value": "GiA2qUishZ7cDwH/j2a9xcqqusSg1jKRnPux6XRxc5rvdRAB",
            "keyMaterialType": "SYMMETRIC"
          },
          "status": "ENABLED",
          "keyId": 872908418,
          "outputPrefixType": "TINK"
        }
      ]
    }"""

KEYSET_WITH_TWO_KEYS = (
    '{"primaryKeyId":282600252,"key":['
    '{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtEcdsaPublicKey",'
    '"value":"EAEaIBDPI66hjLHvjxmUJ2nyHIBDmdOtQ4gPsvWgYYgZ0gygIiBTEK0rTACpAb97m'
    '+mvtJKAk0'
    'q3mHjPcUZm0C4EueDW4Q==",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":282600252,"outputPrefixType":"RAW"},'
    '{"keyData":{'
    '"typeUrl":"type.googleapis.com/google.crypto.tink.JwtRsaSsaPkcs1PublicKey"'
    ',"value":"EAEagQIAkspk37lGBqXmPPq2CL5KdDeRx7xFiTadpL3jc4nXaqftCtpM6qExfrc2'
    'JLaIsnwpwfGMClfe/alIs2GrT9fpM8oDeCccvC39DzZhsSFnAELggi3hnWNKRLfSV0UJzBI+5h'
    'Z6ifUsv8W8mSHKlsVMmvOfC2P5+l72qTwN6Le3hy6CxFp5s9pw011B7J3PU65sty6GI9sehB2B'
    '/n7nfiWw9YN5++pfwyoitzoMoVKOOpj7fFq88f8ArpC7kR1SBTe20Bt1AmpZDT2Dmfmlb/Q1UF'
    'jj/F3C77NCNQ344ZcAEI42HY+uighy5GdKQRHMoTT1OzyDG90ABjggQqDGW+zXzyIDAQAB",'
    '"keyMaterialType":"ASYMMETRIC_PUBLIC"'
    '},"status":"ENABLED","keyId":482168993,"outputPrefixType":"RAW"}]}')

JWK_SET_WITH_TWO_KEYS = (
    '{"keys":[{'
    '"kty":"EC",'
    '"crv":"P-256",'
    '"x":"EM8jrqGMse-PGZQnafIcgEOZ061DiA-y9aBhiBnSDKA",'
    '"y":"UxCtK0wAqQG_e5vpr7SSgJNKt5h4z3FGZtAuBLng1uE",'
    '"use":"sig","alg":"ES256","key_ops":["verify"]},'
    '{"kty":"RSA",'
    '"n":"AJLKZN-5Rgal5jz6tgi-SnQ3kce8RYk2naS943OJ12qn7QraTOqhMX63NiS2iLJ8KcHxj'
    'ApX3v2pSLNhq0_X6TPKA3gnHLwt_Q82YbEhZwBC4IIt4Z1jSkS30ldFCcwSPuYWeon1LL_FvJk'
    'hypbFTJrznwtj-fpe9qk8Dei3t4cugsRaebPacNNdQeydz1OubLcuhiPbHoQdgf5-534lsPWDe'
    'fvqX8MqIrc6DKFSjjqY-3xavPH_AK6Qu5EdUgU3ttAbdQJqWQ09g5n5pW_0NVBY4_xdwu-zQjU'
    'N-OGXABCONh2ProoIcuRnSkERzKE09Ts8gxvdAAY4IEKgxlvs188",'
    '"e":"AQAB","use":"sig","alg":"RS256","key_ops":["verify"]}]}')


class JwkSetConverterTest(parameterized.TestCase):

  @parameterized.named_parameters([
      ('ES256_RAW', ES256_KEYSET, ES256_JWK_SET),
      ('ES384_RAW', ES384_KEYSET, ES384_JWK_SET),
      ('ES512_RAW', ES512_KEYSET, ES512_JWK_SET),
      ('WITH_DISABLED_KEY', KEYSET_WITH_DISABLED_KEY, ES256_JWK_SET),
      ('ES256_TINK', ES256_KEYSET_TINK, ES256_JWK_SET_KID),
      ('RS256_RAW', RS256_KEYSET, RS256_JWK_SET),
      ('RS384_RAW', RS384_KEYSET, RS384_JWK_SET),
      ('RS512_RAW', RS512_KEYSET, RS512_JWK_SET),
      ('RS256_TINK', RS256_KEYSET_TINK, RS256_JWK_SET_KID),
      ('PS256_RAW', PS256_KEYSET, PS256_JWK_SET),
      ('PS384_RAW', PS384_KEYSET, PS384_JWK_SET),
      ('PS512_RAW', PS512_KEYSET, PS512_JWK_SET),
      ('PS256_TINK', PS256_KEYSET_TINK, PS256_JWK_SET_KID),
      ('TWO_KEYS', KEYSET_WITH_TWO_KEYS, JWK_SET_WITH_TWO_KEYS)
  ])
  def test_convert_from_jwt_key(self, tink_keyset, expected_jwk_set):
    keyset_handle = tink.json_proto_keyset_format.parse_without_secret(
        tink_keyset
    )
    jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(jwk_set, expected_jwk_set)

  @parameterized.named_parameters([('ES256_RAW', ES256_JWK_SET),
                                   ('ES384_RAW', ES384_JWK_SET),
                                   ('ES512_RAW', ES512_JWK_SET),
                                   ('ES256_TINK', ES256_JWK_SET_KID),
                                   ('RS256_RAW', RS256_JWK_SET),
                                   ('RS384_RAW', RS384_JWK_SET),
                                   ('RS512_RAW', RS512_JWK_SET),
                                   ('RS256_TINK', RS256_JWK_SET_KID),
                                   ('PS256_RAW', RS256_JWK_SET),
                                   ('PS384_RAW', RS384_JWK_SET),
                                   ('PS512_RAW', RS512_JWK_SET),
                                   ('PS256_TINK', RS256_JWK_SET_KID)])
  def test_convert_jwk_set_to_public_keyset_handle_and_back(self, jwk_set):
    keyset_handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    output_jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(output_jwk_set, jwk_set)
    # check that all keys are raw.
    for key in keyset_handle._keyset.key:
      self.assertEqual(key.output_prefix_type, tink_pb2.RAW)

    # test deprecated to/from keyset_handle functions.
    self.assertEqual(
        jwt.jwk_set_from_keyset_handle(jwt.jwk_set_to_keyset_handle(jwk_set)),
        jwk_set)

  def test_es_conserves_empty_kid(self):
    jwk_set_with_empty_kid = ES256_JWK_SET_KID.replace('"ENgjPA"', '""')
    keyset_handle = jwt.jwk_set_to_public_keyset_handle(jwk_set_with_empty_kid)
    output_jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(output_jwk_set, jwk_set_with_empty_kid)

  def test_primary_key_id_missing_success(self):
    keyset = ES256_KEYSET.replace('"primaryKeyId":282600252,', '')
    keyset_handle = tink.json_proto_keyset_format.parse_without_secret(keyset)
    jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(jwk_set, ES256_JWK_SET)

  @parameterized.named_parameters([
      ('ES256_RAW', ES256_KEYSET),
      ('RS256_RAW', RS256_KEYSET),
      ('PS256_RAW', PS256_KEYSET)
  ])
  def test_from_legacy_ecdsa_keyset_fails(self, keyset):
    legacy_keyset = keyset.replace('RAW', 'LEGACY')
    keyset_handle = tink.json_proto_keyset_format.parse_without_secret(
        legacy_keyset
    )
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_from_public_keyset_handle(keyset_handle)

  @parameterized.named_parameters([
      ('ES256_RAW', ES256_KEYSET),
      ('RS256_RAW', RS256_KEYSET),
      ('PS256_RAW', RS256_KEYSET)
  ])
  def test_from_crunchy_ecdsa_keyset_fails(self, keyset):
    crunchy_keyset = keyset.replace('RAW', 'CRUNCHY')
    keyset_handle = tink.json_proto_keyset_format.parse_without_secret(
        crunchy_keyset
    )
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_from_public_keyset_handle(keyset_handle)

  def test_from_hs256_keyset_fails(self):
    keyset_handle = tink.json_proto_keyset_format.parse(
        HS256_KEYSET, secret_key_access.TOKEN
    )
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_from_public_keyset_handle(keyset_handle)

  def test_from_private_keyset_fails(self):
    keyset_handle = tink.json_proto_keyset_format.parse(
        PRIVATEKEY_KEYSET, secret_key_access.TOKEN
    )
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_from_public_keyset_handle(keyset_handle)

  def test_ecdsa_without_use_or_key_ops_to_public_keyset_handle_success(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256"
        }]}"""
    # ignore returned value, we only test that it worked.
    jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_private_key_to_public_keyset_handle_fails(self):
    # Example from https://datatracker.ietf.org/doc/html/rfc7517#appendix-A.2
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"MKBCTNIcKUSDii11ySs3526iDZ8AiTo7Tu6KPAqv7D4",
           "y":"4Etl6SRW2YiLUrN5vfvVHuhp7x8PxltmWWlbbM4IFyM",
           "d":"870MB6gfuTJ4HtUnUvYMyJpr5eUZNP4Bk43bVdj3eAE",
           "alg":"ES256"
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_with_unknown_field_to_public_keyset_handle_success(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "unknown":1234,
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_without_alg_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_without_kty_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_without_crv_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_with_small_x_primitive_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"AAAwOQ",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeyVerify)

  def test_ecdsa_key_with_small_y_primitive_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"AAAwOQ",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeyVerify)

  def test_ecdsa_key_with_invalid_kty_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"RSA",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_with_invalid_crv_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-384",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["verify"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_with_invalid_use_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"invalid",
           "key_ops":["verify"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_with_invalid_key_ops_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":["invalid"]
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_ecdsa_key_with_string_key_ops_to_public_keyset_handle_fails(self):
    jwk_set = """{"keys":[
        {
           "kty":"EC",
           "crv":"P-256",
           "x":"KUPydf4k4cS5EGS82npjEUxKIiBfUGP3wlN49A2GxTY",
           "y":"b22m_Y4sT-jUJSxBVqjrW_DxWyBLopxYHTuFVfx70ZI",
           "alg":"ES256",
           "use":"sig",
           "key_ops":"verify"
        }]}"""
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_ssa_pkcs1_without_use_and_key_ops_to_keyset_handle_success(self):
    jwk_set = RS256_JWK_SET.replace(',"use":"sig"',
                                    '').replace(',"key_ops":["verify"]', '')
    keyset_handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    output_jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(output_jwk_set, RS256_JWK_SET)

  def test_rsa_ssa_pss_without_use_and_key_ops_to_keyset_handle_success(self):
    jwk_set = PS256_JWK_SET.replace(',"use":"sig"',
                                    '').replace(',"key_ops":["verify"]', '')
    keyset_handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    output_jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(output_jwk_set, PS256_JWK_SET)

  def test_rsa_ssa_pkcs1_with_unknown_property_keyset_handle_success(self):
    jwk_set = RS256_JWK_SET.replace(',"use":"sig"',
                                    ',"use":"sig","unknown":1234')
    keyset_handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    output_jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(output_jwk_set, RS256_JWK_SET)

  def test_rsa_ssa_pss_with_unknown_property_keyset_handle_success(self):
    jwk_set = PS256_JWK_SET.replace(',"use":"sig"',
                                    ',"use":"sig","unknown":1234')
    keyset_handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    output_jwk_set = jwt.jwk_set_from_public_keyset_handle(keyset_handle)
    self.assertEqual(output_jwk_set, PS256_JWK_SET)

  def test_rsa_private_key_to_public_keyset_handle_fails(self):
    # Example from https://datatracker.ietf.org/doc/html/rfc7517#appendix-A.2
    jwk_set = """
     {"keys":
       [
         {"kty":"RSA",
          "n":"0vx7agoebGcQSuuPiLJXZptN9nndrQmbXEps2aiAFbWhM78LhWx4
     cbbfAAtVT86zwu1RK7aPFFxuhDR1L6tSoc_BJECPebWKRXjBZCiFV4n3oknjhMst
     n64tZ_2W-5JsGY4Hc5n9yBXArwl93lqt7_RN5w6Cf0h4QyQ5v-65YGjQR0_FDW2Q
     vzqY368QQMicAtaSqzs8KJZgnYb9c7d0zgdAZHzu6qMQvRL5hajrn1n91CbOpbIS
     D08qNLyrdkt-bFTWhAI4vMQFh6WeZu0fM4lFd2NcRwr3XPksINHaQ-G_xBniIqbw
     0Ls1jF44-csFCur-kEgU8awapJzKnqDKgw",
          "e":"AQAB",
          "d":"X4cTteJY_gn4FYPsXB8rdXix5vwsg1FLN5E3EaG6RJoVH-HLLKD9
     M7dx5oo7GURknchnrRweUkC7hT5fJLM0WbFAKNLWY2vv7B6NqXSzUvxT0_YSfqij
     wp3RTzlBaCxWp4doFk5N2o8Gy_nHNKroADIkJ46pRUohsXywbReAdYaMwFs9tv8d
     _cPVY3i07a3t8MN6TNwm0dSawm9v47UiCl3Sk5ZiG7xojPLu4sbg1U2jx4IBTNBz
     nbJSzFHK66jT8bgkuqsk0GjskDJk19Z4qwjwbsnn4j2WBii3RL-Us2lGVkY8fkFz
     me1z0HbIkfz0Y6mqnOYtqc0X4jfcKoAC8Q",
          "p":"83i-7IvMGXoMXCskv73TKr8637FiO7Z27zv8oj6pbWUQyLPQBQxtPV
     nwD20R-60eTDmD2ujnMt5PoqMrm8RfmNhVWDtjjMmCMjOpSXicFHj7XOuVIYQyqV
     WlWEh6dN36GVZYk93N8Bc9vY41xy8B9RzzOGVQzXvNEvn7O0nVbfs",
          "q":"3dfOR9cuYq-0S-mkFLzgItgMEfFzB2q3hWehMuG0oCuqnb3vobLyum
     qjVZQO1dIrdwgTnCdpYzBcOfW5r370AFXjiWft_NGEiovonizhKpo9VVS78TzFgx
     kIdrecRezsZ-1kYd_s1qDbxtkDEgfAITAG9LUnADun4vIcb6yelxk",
          "dp":"G4sPXkc6Ya9y8oJW9_ILj4xuppu0lzi_H7VTkS8xj5SdX3coE0oim
     YwxIi2emTAue0UOa5dpgFGyBJ4c8tQ2VF402XRugKDTP8akYhFo5tAA77Qe_Nmtu
     YZc3C3m3I24G2GvR5sSDxUyAN2zq8Lfn9EUms6rY3Ob8YeiKkTiBj0",
          "dq":"s9lAH9fggBsoFR8Oac2R_E2gw282rT2kGOAhvIllETE1efrA6huUU
     vMfBcMpn8lqeW6vzznYY5SSQF7pMdC_agI3nG8Ibp1BUb0JUiraRNqUfLhcQb_d9
     GF4Dh7e74WbRsobRonujTYN1xCaP6TO61jvWrX-L18txXw494Q_cgk",
          "qi":"GyM_p6JrXySiz1toFgKbWV-JdI3jQ4ypu9rbMWx3rQJBfmt0FoYzg
     UIZEVFEcOqwemRN81zoDAaa-Bk0KWNGDjJHZDdDmFhW3AN7lI-puxk_mHZGJ11rx
     yR8O55XLSe3SPmRfKwZI6yU24ZxvQKFYItdldUKGzO6Ia6zTKhAVRU",
          "alg":"RS256",
          "kid":"2011-04-29"}
       ]
     }"""
    # remove spaces and line breaks
    jwk_set = jwk_set.replace(' ', '').replace('\n', '')
    # PKCS1
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)
    # PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_without_alg_fails(self):
    jwk_set = RS256_JWK_SET.replace(',"alg":"RS256"', '')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_without_kty_fails(self):
    jwk_set = RS256_JWK_SET.replace('"kty":"RSA",', '')
    # PKCS1
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)
    # PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_with_small_n_primitive_fails(self):
    jwk_set = """{"keys":[{
        "kty":"RSA",
        "n":"AAAwOQ",
        "e":"AQAB",
        "use":"sig",
        "alg":"RS256",
        "key_ops":["verify"]}]}"""
    # PKCS1
    handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeyVerify)
    # test PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    handle = jwt.jwk_set_to_public_keyset_handle(jwk_set)
    with self.assertRaises(tink.TinkError):
      handle.primitive(jwt.JwtPublicKeyVerify)

  def test_rsa_with_invalid_kty_fails(self):
    jwk_set = RS256_JWK_SET.replace('"kty":"RSA"', '"kty":"EC"')
    # PKCS1
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)
    # PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_with_invalid_use_fails(self):
    jwk_set = RS256_JWK_SET.replace('"use":"sig"', '"use":"invalid"')
    # PKCS1
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)
    # PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_with_invalid_key_ops_fails(self):
    jwk_set = RS256_JWK_SET.replace('"key_ops":["verify"]',
                                    '"key_ops":["invalid"]')
    # PKCS1
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)
    # PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_rsa_with_string_key_ops_fails(self):
    jwk_set = RS256_JWK_SET.replace('"key_ops":["verify"]',
                                    '"key_ops":"verify"')
    # PKCS1
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)
    # PSS
    jwk_set = jwk_set.replace('"alg":"RS256"', '"alg":"PS256"')
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle(jwk_set)

  def test_jwk_set_to_public_keyset_handle_with_invalid_json_raises_tink_error(
      self):
    with self.assertRaises(tink.TinkError):
      jwt.jwk_set_to_public_keyset_handle('invalid')

if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
"""Tests for tink.python.tink.jwt._verified_jwt."""

import datetime

from absl.testing import absltest
from tink import jwt

ISSUED_AT = datetime.datetime.fromtimestamp(1582230020, datetime.timezone.utc)
NOT_BEFORE = datetime.datetime.fromtimestamp(1893553445, datetime.timezone.utc)
EXPIRATION = datetime.datetime.fromtimestamp(2218027244, datetime.timezone.utc)


class VerifiedJwtTest(absltest.TestCase):

  def test_empty(self):
    token = jwt.VerifiedJwt._create(jwt.new_raw_jwt(without_expiration=True))
    with self.assertRaises(KeyError):
      token.type_header()
    with self.assertRaises(KeyError):
      token.issuer()
    with self.assertRaises(KeyError):
      token.subject()
    with self.assertRaises(KeyError):
      token.jwt_id()
    with self.assertRaises(KeyError):
      token.audiences()
    with self.assertRaises(KeyError):
      token.expiration()
    with self.assertRaises(KeyError):
      token.issued_at()
    with self.assertRaises(KeyError):
      token.not_before()
    with self.assertRaises(KeyError):
      token.custom_claim('unknown')
    self.assertFalse(token.has_issuer())
    self.assertFalse(token.has_subject())
    self.assertFalse(token.has_jwt_id())
    self.assertFalse(token.has_audiences())
    self.assertFalse(token.has_expiration())
    self.assertFalse(token.has_issued_at())
    self.assertFalse(token.has_not_before())

  def test_full(self):
    token = jwt.VerifiedJwt._create(
        jwt.new_raw_jwt(
            type_header='TypeHeader',
            issuer='Issuer',
            subject='Subject',
            jwt_id='JWT ID',
            audiences=['bob', 'eve'],
            expiration=EXPIRATION,
            issued_at=ISSUED_AT,
            not_before=NOT_BEFORE))
    self.assertTrue(token.has_type_header())
    self.assertEqual(token.type_header(), 'TypeHeader')
    self.assertTrue(token.has_issuer())
    self.assertEqual(token.issuer(), 'Issuer')
    self.assertTrue(token.has_subject())
    self.assertEqual(token.subject(), 'Subject')
    self.assertTrue(token.has_jwt_id())
    self.assertEqual(token.jwt_id(), 'JWT ID')
    self.assertTrue(token.has_audiences())
    self.assertEqual(token.audiences(), ['bob', 'eve'])
    self.assertTrue(token.has_expiration())
    self.assertEqual(token.expiration(), EXPIRATION)
    self.assertTrue(token.has_issued_at())
    self.assertEqual(token.issued_at(), ISSUED_AT)
    self.assertTrue(token.has_not_before())
    self.assertEqual(token.not_before(), NOT_BEFORE)

  def test_custom_claims(self):
    custom_claims = {'string': 'value',
                     'boolean': True,
                     'number': 123.456,
                     'integer': 123,
                     'null': None,
                     'array': [1, None, 'Bob', 2.2, {'foo': 'bar'}],
                     'object': {'one': {'two': 3}}}
    token = token = jwt.VerifiedJwt._create(
        jwt.new_raw_jwt(custom_claims=custom_claims, without_expiration=True))
    self.assertCountEqual(
        token.custom_claim_names(),
        {'string', 'boolean', 'number', 'integer', 'null', 'array', 'object'})
    self.assertEqual(token.custom_claim('string'), 'value')
    self.assertEqual(token.custom_claim('boolean'), True)
    self.assertEqual(token.custom_claim('number'), 123.456)
    self.assertEqual(token.custom_claim('integer'), 123)
    self.assertIsNone(token.custom_claim('null'))
    self.assertEqual(
        token.custom_claim('array'),
        [1, None, 'Bob', 2.2, {'foo': 'bar'}])
    self.assertEqual(token.custom_claim('object'), {'one': {'two': 3}})


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
"""Tests for tink.python.tink.jwt._raw_jwt."""

import datetime
import json
import sys

from typing import cast, Dict, List

from absl.testing import absltest
# from absl.testing import parameterized

from tink import jwt

ISSUED_AT_TIMESTAMP = 1582230020
ISSUED_AT = datetime.datetime.fromtimestamp(ISSUED_AT_TIMESTAMP,
                                            datetime.timezone.utc)

NOT_BEFORE_TIMESTAMP = 1893553445
NOT_BEFORE = datetime.datetime.fromtimestamp(NOT_BEFORE_TIMESTAMP,
                                             datetime.timezone.utc)

EXPIRATION_TIMESTAMP = 2218027244
EXPIRATION = datetime.datetime.fromtimestamp(EXPIRATION_TIMESTAMP,
                                             datetime.timezone.utc)


class RawJwtTest(absltest.TestCase):

  def test_datetime(self):
    t = 1893553445
    d = datetime.datetime.fromtimestamp(t, datetime.timezone.utc)
    self.assertEqual(int(d.timestamp()), t)

  def test_empty_access_raises_error(self):
    token = jwt.new_raw_jwt(without_expiration=True)
    with self.assertRaises(KeyError):
      token.type_header()
    with self.assertRaises(KeyError):
      token.issuer()
    with self.assertRaises(KeyError):
      token.subject()
    with self.assertRaises(KeyError):
      token.jwt_id()
    with self.assertRaises(KeyError):
      token.audiences()
    with self.assertRaises(KeyError):
      token.expiration()
    with self.assertRaises(KeyError):
      token.issued_at()
    with self.assertRaises(KeyError):
      token.not_before()
    with self.assertRaises(KeyError):
      token.custom_claim('unknown')

  def test_empty_has_returns_false(self):
    token = jwt.new_raw_jwt(without_expiration=True)
    self.assertFalse(token.has_type_header())
    self.assertFalse(token.has_issuer())
    self.assertFalse(token.has_subject())
    self.assertFalse(token.has_jwt_id())
    self.assertFalse(token.has_audiences())
    self.assertFalse(token.has_expiration())
    self.assertFalse(token.has_issued_at())
    self.assertFalse(token.has_not_before())

  def test_type_header(self):
    token = jwt.new_raw_jwt(type_header='TypeHeader', without_expiration=True)
    self.assertTrue(token.has_type_header())
    self.assertEqual(token.type_header(), 'TypeHeader')

  def test_issuer(self):
    token = jwt.new_raw_jwt(issuer='Issuer', without_expiration=True)
    self.assertTrue(token.has_issuer())
    self.assertEqual(token.issuer(), 'Issuer')

  def test_subject(self):
    token = jwt.new_raw_jwt(subject='Subject', without_expiration=True)
    self.assertTrue(token.has_subject())
    self.assertEqual(token.subject(), 'Subject')

  def test_jwt_id(self):
    token = jwt.new_raw_jwt(jwt_id='JWT ID', without_expiration=True)
    self.assertTrue(token.has_jwt_id())
    self.assertEqual(token.jwt_id(), 'JWT ID')

  def test_audience(self):
    token = jwt.new_raw_jwt(audience='bob', without_expiration=True)
    self.assertTrue(token.has_audiences())
    self.assertEqual(token.audiences(), ['bob'])

  def test_audiences(self):
    token = jwt.new_raw_jwt(audiences=['bob', 'eve'], without_expiration=True)
    self.assertTrue(token.has_audiences())
    self.assertEqual(token.audiences(), ['bob', 'eve'])

  def test_string_audiences(self):
    token = jwt.new_raw_jwt(
        audiences=cast(List[str], 'bob'), without_expiration=True)
    self.assertTrue(token.has_audiences())
    self.assertEqual(token.audiences(), ['bob'])

  def test_empty_audiences(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(audiences=[], without_expiration=True)

  def test_expiration(self):
    token = jwt.new_raw_jwt(expiration=EXPIRATION)
    self.assertTrue(token.has_expiration())
    self.assertEqual(token.expiration(), EXPIRATION)

  def test_round_down_expiration_with_fraction(self):
    token = jwt.new_raw_jwt(
        expiration=datetime.datetime.fromtimestamp(123.999,
                                                   datetime.timezone.utc))
    self.assertEqual(
        token.expiration(),
        datetime.datetime.fromtimestamp(123, datetime.timezone.utc))

  @absltest.skipIf(
      sys.platform.startswith('win32'),
      'Windows does not allow `localtime` values after 23:59:59, December 31,'
      ' 3000, UTC',
  )
  def test_large_timestamps_success(self):
    # year 9999
    large = datetime.datetime.fromtimestamp(253402300799,
                                            datetime.timezone.utc)
    token = jwt.new_raw_jwt(
        expiration=large, issued_at=large, not_before=large)
    self.assertTrue(token.has_expiration())
    self.assertTrue(token.has_issued_at())
    self.assertTrue(token.has_not_before())
    self.assertEqual(token.expiration(), large)
    self.assertEqual(token.issued_at(), large)
    self.assertEqual(token.not_before(), large)

  @absltest.skipIf(
      sys.platform.startswith('win32'),
      'Windows does not allow `localtime` values after 23:59:59, December 31,'
      ' 3000, UTC',
  )
  def test_too_large_timestamps_fail(self):
    with self.assertRaises(ValueError):
      datetime.datetime.fromtimestamp(253402300800, datetime.timezone.utc)

  def test_negative_expiration_fails(self):
    neg = datetime.datetime.fromtimestamp(-1.0, datetime.timezone.utc)
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(expiration=neg)
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(issued_at=neg, without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(not_before=neg, without_expiration=True)

  def test_issued_at(self):
    token = jwt.new_raw_jwt(issued_at=ISSUED_AT, without_expiration=True)
    self.assertTrue(token.has_issued_at())
    self.assertEqual(token.issued_at(), ISSUED_AT)

  def test_round_down_issued_at_with_fraction(self):
    token = jwt.new_raw_jwt(
        without_expiration=True,
        issued_at=datetime.datetime.fromtimestamp(123.999,
                                                  datetime.timezone.utc))
    self.assertEqual(
        token.issued_at(),
        datetime.datetime.fromtimestamp(123, datetime.timezone.utc))

  def test_not_before(self):
    token = jwt.new_raw_jwt(not_before=NOT_BEFORE, without_expiration=True)
    self.assertTrue(token.has_not_before())
    self.assertEqual(token.not_before(), NOT_BEFORE)

  def test_round_down_not_before_with_fraction(self):
    token = jwt.new_raw_jwt(
        without_expiration=True,
        not_before=datetime.datetime.fromtimestamp(123.999,
                                                   datetime.timezone.utc))
    self.assertEqual(
        token.not_before(),
        datetime.datetime.fromtimestamp(123, datetime.timezone.utc))

  def test_rejects_naive_time(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(
          issued_at=ISSUED_AT.replace(tzinfo=None), without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(
          not_before=NOT_BEFORE.replace(tzinfo=None), without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(expiration=EXPIRATION.replace(tzinfo=None))

  def test_custom_claims(self):
    custom_claims = {'string': 'value',
                     'boolean': True,
                     'number': 123.456,
                     'integer': 123,
                     'null': None,
                     'array': [1, None, 'Bob', 2.2, {'foo': 'bar'}],
                     'object': {'one': {'two': 3}}}
    token = jwt.new_raw_jwt(
        custom_claims=custom_claims, without_expiration=True)
    self.assertCountEqual(
        token.custom_claim_names(),
        {'string', 'boolean', 'number', 'integer', 'null', 'array', 'object'})
    self.assertEqual(token.custom_claim('string'), 'value')
    self.assertEqual(token.custom_claim('boolean'), True)
    self.assertEqual(token.custom_claim('number'), 123.456)
    self.assertEqual(token.custom_claim('integer'), 123)
    self.assertIsNone(token.custom_claim('null'))
    self.assertEqual(token.custom_claim('array'),
                     [1, None, 'Bob', 2.2, {'foo': 'bar'}])
    self.assertEqual(token.custom_claim('object'), {'one': {'two': 3}})

  def test_empty_custom_claim_names(self):
    token = jwt.new_raw_jwt(without_expiration=True)
    self.assertEmpty(token.custom_claim_names())

  def test_registered_name_as_custom_claim_is_invalid(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.new_raw_jwt(custom_claims={'iss': 'issuer'}, without_expiration=True)

  def test_custom_claim_using_registered_name_fails(self):
    token = jwt.new_raw_jwt(issuer='issuer', without_expiration=True)
    with self.assertRaises(jwt.JwtInvalidError):
      token.custom_claim('iss')

  def test_null_custom_claim(self):
    token = jwt.new_raw_jwt(
        custom_claims={'null_claim': None}, without_expiration=True)
    self.assertCountEqual(token.custom_claim_names(), ['null_claim'])
    self.assertIsNone(token.custom_claim('null_claim'))

  def test_payload(self):
    custom_claims = {'account_no': 1234,
                     'amount': 1.5,
                     'name': 'Peter',
                     'null': None,
                     'array': [1, None, 'Bob', 2.2, {'foo': 'bar'}],
                     'object': {'one': {'two': 3}}
                     }
    token = jwt.new_raw_jwt(
        issuer='Issuer',
        subject='Subject',
        audiences=['bob', 'eve'],
        jwt_id='JWT ID',
        issued_at=ISSUED_AT,
        not_before=NOT_BEFORE,
        expiration=EXPIRATION,
        custom_claims=custom_claims)
    self.assertEqual(
        json.loads(token.json_payload()), {
            'iss': 'Issuer',
            'sub': 'Subject',
            'jti': 'JWT ID',
            'aud': ['bob', 'eve'],
            'iat': ISSUED_AT_TIMESTAMP,
            'nbf': NOT_BEFORE_TIMESTAMP,
            'exp': EXPIRATION_TIMESTAMP,
            'account_no': 1234,
            'amount': 1.5,
            'name': 'Peter',
            'null': None,
            'array': [1, None, 'Bob', 2.2, {'foo': 'bar'}],
            'object': {'one': {'two': 3}}
        })

  def test_string_audience_payload(self):
    token = jwt.new_raw_jwt(audience='bob', without_expiration=True)
    self.assertEqual(json.loads(token.json_payload()), {'aud': 'bob'})

  def test_from_to_payload(self):
    payload = {
        'iss': 'Issuer',
        'sub': 'Subject',
        'jti': 'JWT ID',
        'aud': ['bob', 'eve'],
        'iat': ISSUED_AT_TIMESTAMP,
        'nbf': NOT_BEFORE_TIMESTAMP,
        'exp': EXPIRATION_TIMESTAMP,
        'account_no': 1234,
        'amount': 1.5,
        'name': 'Peter',
        'null': None,
        'array': [1, None, 'Bob', 2.2, {'foo': 'bar'}],
        'object': {'one': {'two': 3}}
    }
    token = jwt.RawJwt._from_json(None, json.dumps(payload))
    self.assertEqual(json.loads(token.json_payload()), payload)

  def test_integer_is_encoded_as_integer(self):
    token = jwt.new_raw_jwt(
        without_expiration=True,
        custom_claims={'num': 1})
    self.assertEqual(token.json_payload(), '{"num":1}')

  def test_exp_to_payload(self):
    expiration = datetime.datetime.fromtimestamp(2218027244,
                                                 datetime.timezone.utc)
    token = jwt.new_raw_jwt(expiration=expiration)
    self.assertEqual(token.json_payload(), '{"exp":2218027244}')

  def test_float_exp_to_payload(self):
    expiration = datetime.datetime.fromtimestamp(123.999, datetime.timezone.utc)
    token = jwt.new_raw_jwt(expiration=expiration)
    self.assertEqual(token.json_payload(), '{"exp":123}')

  def test_from_to_payload_with_string_audience_perserves_string(self):
    payload = {
        'iss': 'Issuer',
        'aud': 'bob',
    }
    token = jwt.RawJwt._from_json(None, json.dumps(payload))
    expected = {
        'iss': 'Issuer',
        'aud': 'bob',
    }
    self.assertEqual(json.loads(token.json_payload()), expected)

  def test_from_to_payload_with_list_audience_perserves_list(self):
    payload = {
        'iss': 'Issuer',
        'aud': ['bob'],
    }
    token = jwt.RawJwt._from_json(None, json.dumps(payload))
    expected = {
        'iss': 'Issuer',
        'aud': ['bob'],
    }
    self.assertEqual(json.loads(token.json_payload()), expected)

  def test_from_payload_with_wrong_issuer_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"iss":123}')

  def test_from_payload_with_wrong_subject_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"sub":123}')

  def test_from_payload_with_wrong_jwt_id_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"jti":123}')

  def test_from_payload_with_wrong_expiration_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"exp":"123"}')

  def test_from_payload_with_wrong_issued_at_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"iat":"123"}')

  def test_from_payload_with_wrong_not_before_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"nbf":"123"}')

  def test_from_payload_with_exp_expiration_success(self):
    token = jwt.RawJwt._from_json(None, '{"exp":1e10}')
    self.assertEqual(
        token.expiration(),
        datetime.datetime.fromtimestamp(10000000000, datetime.timezone.utc))

  def test_from_payload_with_large_expiration_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"exp":1e30}')

  def test_from_payload_with_negative_expiration_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"exp":-1}')

  def test_from_payload_with_infinity_expiration_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"exp":Infinity}')

  def test_from_payload_with_utf16_surrogate(self):
    # the json string contains "\uD834\uDD1E", which the JSON decoder decodes to
    # the G clef character (U+1D11E).
    token = jwt.RawJwt._from_json(None, u'{"iss":"\\uD834\\uDD1E"}')
    self.assertEqual(token.issuer(), u'\U0001d11e')

  def test_from_payload_with_invalid_utf16(self):
    # the json string contains "\uD834", which gets decoded by the json decoder
    # into an invalid UTF16 character.
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, u'{"iss":"\\uD834"}')
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, u'{"\\uD834":"issuer"}')
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, u'{"a":{"a":{"a":"\\uD834"}}}')

  def test_from_payload_with_duplicate_map_keys_fails(self):
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"claim": "claim1", "claim": "claim2"}')
    with self.assertRaises(jwt.JwtInvalidError):
      jwt.RawJwt._from_json(None, '{"nested": {"a: "a1", "a": "a2"}}')
    # this is fine, since the two 'a' keys are not in the same map
    _ = jwt.RawJwt._from_json(None, '{"a": "a1", "b": {"a": "a2"}}')

  def test_modification(self):
    audiences = ['alice', 'bob']
    my_claim = {'one': 'two'}
    custom_claims = {'my_claim': my_claim}
    token = jwt.new_raw_jwt(
        issuer='Issuer',
        audiences=audiences,
        custom_claims=custom_claims,
        without_expiration=True)

    # modify inputs and outputs of token
    audiences.append('eve')
    custom_claims['new_claim'] = 456
    my_claim['three'] = 4
    output_claim = cast(Dict[str, str], token.custom_claim('my_claim'))
    output_claim['three'] = 4  # pytype: disable=container-type-mismatch

    # modifications don't affect token.
    self.assertEqual(token.audiences(), ['alice', 'bob'])
    self.assertEqual(token.custom_claim_names(), {'my_claim'})
    self.assertEqual(token.custom_claim('my_claim'), {'one': 'two'})
    with self.assertRaises(KeyError):
      token.custom_claim('new_claim')


if __name__ == '__main__':
  absltest.main()



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A JWT HMAC key manager."""

from typing import Optional, Type

from tink.proto import jwt_hmac_pb2
from tink.proto import tink_pb2
from tink import core
from tink.cc.pybind import tink_bindings
from tink.jwt import _json_util
from tink.jwt import _jwt_error
from tink.jwt import _jwt_format
from tink.jwt import _jwt_mac
from tink.jwt import _jwt_validator
from tink.jwt import _raw_jwt
from tink.jwt import _verified_jwt

_JWT_HMAC_KEY_TYPE = 'type.googleapis.com/google.crypto.tink.JwtHmacKey'

_ALGORITHM_STRING = {
    jwt_hmac_pb2.HS256: 'HS256',
    jwt_hmac_pb2.HS384: 'HS384',
    jwt_hmac_pb2.HS512: 'HS512'
}


class _JwtHmac(_jwt_mac.JwtMacInternal):
  """Interface for authenticating and verifying JWT with JWS MAC."""

  def __init__(self, cc_mac: tink_bindings.Mac, algorithm: str,
               custom_kid: Optional[str]):
    self._cc_mac = cc_mac
    self._algorithm = algorithm
    self._custom_kid = custom_kid

  @core.use_tink_errors
  def _compute_mac(self, data: bytes) -> bytes:
    return self._cc_mac.compute_mac(data)

  @core.use_tink_errors
  def _verify_mac(self, mac_value: bytes, data: bytes) -> None:
    self._cc_mac.verify_mac(mac_value, data)

  def compute_mac_and_encode_with_kid(self, raw_jwt: _raw_jwt.RawJwt,
                                      kid: Optional[str]) -> str:
    """Computes a MAC and encodes the token.

    Args:
      raw_jwt: The RawJwt token to be MACed and encoded.
      kid: Optional "kid" header value. It is set by the wrapper for keys with
        output prefix TINK, and it is None for output prefix RAW.

    Returns:
      The MACed token encoded in the JWS compact serialization format.
    Raises:
      tink.TinkError if the operation fails.
    """
    if self._custom_kid is not None:
      if kid is not None:
        raise _jwt_error.JwtInvalidError(
            'custom_kid must not be set for keys with output prefix type TINK')
      kid = self._custom_kid
    unsigned = _jwt_format.create_unsigned_compact(self._algorithm, kid,
                                                   raw_jwt)
    return _jwt_format.create_signed_compact(unsigned,
                                             self._compute_mac(unsigned))

  def verify_mac_and_decode_with_kid(
      self, compact: str, validator: _jwt_validator.JwtValidator,
      kid: Optional[str]) -> _verified_jwt.VerifiedJwt:
    """Verifies, validates and decodes a MACed compact JWT token."""
    parts = _jwt_format.split_signed_compact(compact)
    unsigned_compact, json_header, json_payload, mac = parts
    self._verify_mac(mac, unsigned_compact)
    header = _json_util.json_loads(json_header)
    _jwt_format.validate_header(
        header=header,
        algorithm=self._algorithm,
        tink_kid=kid,
        custom_kid=self._custom_kid)
    raw_jwt = _raw_jwt.raw_jwt_from_json(
        _jwt_format.get_type_header(header), json_payload)
    _jwt_validator.validate(validator, raw_jwt)
    return _verified_jwt.VerifiedJwt._create(raw_jwt)  # pylint: disable=protected-access


class MacCcToPyJwtMacKeyManager(core.KeyManager[_jwt_mac.JwtMacInternal]):
  """Transforms C++ KeyManager into a Python KeyManager."""

  def __init__(self):
    self._cc_key_manager = tink_bindings.MacKeyManager.from_cc_registry(
        'type.googleapis.com/google.crypto.tink.JwtHmacKey')

  def primitive_class(self) -> Type[_jwt_mac.JwtMacInternal]:
    return _jwt_mac.JwtMacInternal

  @core.use_tink_errors
  def primitive(self, key_data: tink_pb2.KeyData) -> _jwt_mac.JwtMacInternal:
    if key_data.type_url != _JWT_HMAC_KEY_TYPE:
      raise _jwt_error.JwtInvalidError('Invalid key data key type')
    jwt_hmac_key = jwt_hmac_pb2.JwtHmacKey.FromString(key_data.value)
    algorithm = _ALGORITHM_STRING[jwt_hmac_key.algorithm]
    cc_mac = self._cc_key_manager.primitive(key_data.SerializeToString())
    if jwt_hmac_key.HasField('custom_kid'):
      custom_kid = jwt_hmac_key.custom_kid.value
    else:
      custom_kid = None
    return _JwtHmac(cc_mac, algorithm, custom_kid)

  def key_type(self) -> str:
    return self._cc_key_manager.key_type()

  @core.use_tink_errors
  def new_key_data(self,
                   key_template: tink_pb2.KeyTemplate) -> tink_pb2.KeyData:
    data = self._cc_key_manager.new_key_data(key_template.SerializeToString())
    return tink_pb2.KeyData.FromString(data)


def register():
  tink_bindings.register_jwt()
  core.Registry.register_key_manager(
      MacCcToPyJwtMacKeyManager(), new_key_allowed=True)



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for tink.python.tink.core.primitive_set."""

from absl.testing import absltest
from tink.proto import tink_pb2
from tink import aead
from tink import core
from tink import mac
from tink.testing import helper


MAC_TEMPLATE = mac.mac_key_templates.HMAC_SHA256_128BITTAG


def setUpModule():
  mac.register()


def new_key(
    template: tink_pb2.KeyTemplate,
    key_id: int,
    output_prefix_type: tink_pb2.OutputPrefixType = tink_pb2.TINK,
    status: tink_pb2.KeyStatusType = tink_pb2.ENABLED
) -> tink_pb2.Keyset.Key:
  return tink_pb2.Keyset.Key(
      key_data=core.Registry.new_key_data(template),
      key_id=key_id,
      status=status,
      output_prefix_type=output_prefix_type)


class PrimitiveSetTest(absltest.TestCase):

  def test_primitive_returns_entry(self):
    key = new_key(MAC_TEMPLATE, key_id=1)
    primitive = core.Registry.primitive(key.key_data, mac.Mac)
    primitive_set = core.new_primitive_set(mac.Mac)
    primitive_set.add_primitive(primitive, key)
    entries = primitive_set.primitive(key)
    self.assertLen(entries, 1)
    entry = entries[0]
    self.assertEqual(primitive, entry.primitive)
    self.assertEqual(tink_pb2.ENABLED, entry.status)
    self.assertEqual(core.crypto_format.output_prefix(key), entry.identifier)
    self.assertEqual(1, entry.key_id)

  def test_unknown_key_returns_empty_list(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    unknown_key = new_key(MAC_TEMPLATE, key_id=1)
    self.assertEqual(primitive_set.primitive(unknown_key), [])

  def test_primitive_from_identifier_returns_entry(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key = new_key(MAC_TEMPLATE, key_id=1)
    primitive = core.Registry.primitive(key.key_data, mac.Mac)
    primitive_set.add_primitive(primitive, key)

    ident = core.crypto_format.output_prefix(key)
    entries = primitive_set.primitive_from_identifier(ident)
    self.assertLen(entries, 1)
    entry = entries[0]
    self.assertEqual(primitive, entry.primitive)
    self.assertEqual(tink_pb2.ENABLED, entry.status)
    self.assertEqual(ident, entry.identifier)
    self.assertEqual(1, entry.key_id)

  def test_list_of_entries_can_be_modified(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key = new_key(MAC_TEMPLATE, key_id=1)
    primitive = core.Registry.primitive(key.key_data, mac.Mac)
    primitive_set.add_primitive(primitive, key)
    entries = primitive_set.primitive(key)
    entries.append('Something')
    self.assertLen(primitive_set.primitive(key), 1)

  def test_primary_returns_primary(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key = new_key(MAC_TEMPLATE, key_id=1)
    primitive = core.Registry.primitive(key.key_data, mac.Mac)
    entry = primitive_set.add_primitive(primitive, key)
    primitive_set.set_primary(entry)

    entry = primitive_set.primary()
    self.assertEqual(primitive, entry.primitive)
    self.assertEqual(tink_pb2.ENABLED, entry.status)
    self.assertEqual(core.crypto_format.output_prefix(key), entry.identifier)
    self.assertEqual(1, entry.key_id)

  def test_primary_not_set_primary_fails(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key = new_key(MAC_TEMPLATE, key_id=1)
    primitive = core.Registry.primitive(key.key_data, mac.Mac)
    primitive_set.add_primitive(primitive, key)
    with self.assertRaises(core.TinkError):
      primitive_set.primary()

  def test_same_key_id_and_prefix_type(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key1 = new_key(MAC_TEMPLATE, key_id=1)
    primitive1 = core.Registry.primitive(key1.key_data, mac.Mac)
    primitive_set.add_primitive(primitive1, key1)
    key2 = new_key(MAC_TEMPLATE, key_id=1, status=tink_pb2.DISABLED)
    primitive2 = core.Registry.primitive(key2.key_data, mac.Mac)
    primitive_set.add_primitive(primitive2, key2)

    expected_ident = core.crypto_format.output_prefix(key1)
    entries = primitive_set.primitive(key1)
    self.assertLen(entries, 2)
    self.assertEqual(primitive1, entries[0].primitive)
    self.assertEqual(primitive2, entries[1].primitive)
    self.assertEqual(tink_pb2.ENABLED, entries[0].status)
    self.assertEqual(tink_pb2.DISABLED, entries[1].status)
    self.assertEqual(expected_ident, entries[0].identifier)
    self.assertEqual(expected_ident, entries[1].identifier)
    self.assertEqual(1, entries[0].key_id)
    self.assertEqual(1, entries[1].key_id)
    self.assertLen(primitive_set.primitive(key2), 2)

  def test_same_key_id_but_different_prefix_type(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key1 = new_key(MAC_TEMPLATE, key_id=1, output_prefix_type=tink_pb2.TINK)
    primitive1 = core.Registry.primitive(key1.key_data, mac.Mac)
    primitive_set.add_primitive(primitive1, key1)
    key2 = new_key(MAC_TEMPLATE, key_id=1, output_prefix_type=tink_pb2.LEGACY)
    primitive2 = core.Registry.primitive(key2.key_data, mac.Mac)
    primitive_set.add_primitive(primitive2, key2)

    entries1 = primitive_set.primitive(key1)
    self.assertLen(entries1, 1)
    self.assertEqual(primitive1, entries1[0].primitive)
    self.assertEqual(tink_pb2.ENABLED, entries1[0].status)
    self.assertEqual(core.crypto_format.output_prefix(key1),
                     entries1[0].identifier)
    self.assertEqual(1, entries1[0].key_id)

    entries2 = primitive_set.primitive(key2)
    self.assertLen(entries2, 1)
    self.assertEqual(primitive2, entries2[0].primitive)
    self.assertEqual(tink_pb2.ENABLED, entries2[0].status)
    self.assertEqual(core.crypto_format.output_prefix(key2),
                     entries2[0].identifier)
    self.assertEqual(1, entries2[0].key_id)

  def test_add_invalid_key_fails(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key = new_key(MAC_TEMPLATE, key_id=1)
    key.ClearField('output_prefix_type')
    with self.assertRaises(core.TinkError):
      primitive_set.add_primitive(helper.FakeMac(), key)

  def test_add_wrong_primitive_fails(self):
    primitive_set = core.new_primitive_set(aead.Aead)
    key = new_key(MAC_TEMPLATE, key_id=1, output_prefix_type=tink_pb2.TINK)
    primitive = core.Registry.primitive(key.key_data, mac.Mac)
    with self.assertRaises(core.TinkError):
      primitive_set.add_primitive(primitive, key)

  def test_primitive_class(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    self.assertEqual(primitive_set.primitive_class(), mac.Mac)

  def test_raw_primitives(self):
    primitive_set = core.new_primitive_set(mac.Mac)
    key1 = new_key(MAC_TEMPLATE, key_id=1, output_prefix_type=tink_pb2.TINK)
    primitive1 = core.Registry.primitive(key1.key_data, mac.Mac)
    primitive_set.add_primitive(primitive1, key1)
    key2 = new_key(MAC_TEMPLATE, key_id=1, output_prefix_type=tink_pb2.RAW)
    primitive2 = core.Registry.primitive(key2.key_data, mac.Mac)
    primitive_set.add_primitive(primitive2, key2)
    key3 = new_key(
        MAC_TEMPLATE,
        key_id=3,
        output_prefix_type=tink_pb2.RAW,
        status=tink_pb2.DISABLED)
    primitive3 = core.Registry.primitive(key3.key_data, mac.Mac)
    primitive_set.add_primitive(primitive3, key3)

    entries = primitive_set.raw_primitives()
    self.assertLen(entries, 2)
    self.assertEqual(primitive2, entries[0].primitive)
    self.assertEqual(tink_pb2.ENABLED, entries[0].status)
    self.assertEqual(core.crypto_format.RAW_PREFIX,
                     entries[0].identifier)
    self.assertEqual(1, entries[0].key_id)
    self.assertEqual(primitive3, entries[1].primitive)
    self.assertEqual(tink_pb2.DISABLED, entries[1].status)
    self.assertEqual(core.crypto_format.RAW_PREFIX,
                     entries[1].identifier)
    self.assertEqual(3, entries[1].key_id)

  def test_all_primitives(self):
    primitive_set = core.new_primitive_set(mac.Mac)

    key0 = new_key(MAC_TEMPLATE, key_id=88, output_prefix_type=tink_pb2.TINK)
    primitive0 = core.Registry.primitive(key0.key_data, mac.Mac)
    primitive_set.add_primitive(primitive0, key0)

    key1 = new_key(MAC_TEMPLATE, key_id=88, output_prefix_type=tink_pb2.LEGACY)
    primitive1 = core.Registry.primitive(key1.key_data, mac.Mac)
    primitive_set.add_primitive(primitive1, key1)

    key2 = new_key(MAC_TEMPLATE, key_id=88, output_prefix_type=tink_pb2.RAW)
    primitive2 = core.Registry.primitive(key2.key_data, mac.Mac)
    primitive_set.add_primitive(primitive2, key2)

    key3 = new_key(
        MAC_TEMPLATE,
        key_id=89,
        output_prefix_type=tink_pb2.RAW,
        status=tink_pb2.DISABLED)
    primitive3 = core.Registry.primitive(key3.key_data, mac.Mac)
    primitive_set.add_primitive(primitive3, key3)

    key4 = new_key(MAC_TEMPLATE, key_id=88, output_prefix_type=tink_pb2.TINK)
    primitive0 = core.Registry.primitive(key4.key_data, mac.Mac)
    primitive_set.add_primitive(primitive0, key4)

    list_of_entries = primitive_set.all()

    v = []
    for entries in list_of_entries:
      v.append(
          sorted([
              (e.identifier, e.output_prefix_type, e.key_id) for e in entries
          ]))
    self.assertCountEqual(v, [
        [(b'', tink_pb2.RAW, 88), (b'', tink_pb2.RAW, 89)],
        [(b'\x01\x00\x00\x00X', tink_pb2.TINK, 88),
         (b'\x01\x00\x00\x00X', tink_pb2.TINK, 88)],
        [(b'\x00\x00\x00\x00X', tink_pb2.LEGACY, 88)],
    ])

if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink._mac_wrapper."""

from absl.testing import absltest
from absl.testing import parameterized

import tink
from tink import _keyset_handle
from tink import _monitoring
from tink import core
from tink import mac
from tink.testing import fake_key_usage_monitor
from tink.testing import keyset_builder


MAC_TEMPLATE = mac.mac_key_templates.HMAC_SHA256_128BITTAG
RAW_MAC_TEMPLATE = keyset_builder.raw_template(MAC_TEMPLATE)
LEGACY_MAC_TEMPLATE = keyset_builder.legacy_template(MAC_TEMPLATE)


def setUpModule():
  mac.register()


class MacWrapperTest(parameterized.TestCase):

  @parameterized.parameters(
      [MAC_TEMPLATE, RAW_MAC_TEMPLATE, LEGACY_MAC_TEMPLATE]
  )
  def test_compute_verify_mac(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(mac.Mac)
    tag = primitive.compute_mac(b'data')
    # No exception raised, no return value.
    self.assertIsNone(primitive.verify_mac(tag, b'data'))

  @parameterized.parameters(
      [MAC_TEMPLATE, RAW_MAC_TEMPLATE, LEGACY_MAC_TEMPLATE]
  )
  def test_verify_unknown_mac_fails(self, template):
    unknown_handle = tink.new_keyset_handle(template)
    unknown_primitive = unknown_handle.primitive(mac.Mac)
    unknown_tag = unknown_primitive.compute_mac(b'data')

    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(mac.Mac)
    with self.assertRaises(tink.TinkError):
      primitive.verify_mac(unknown_tag, b'data')

  @parameterized.parameters(
      [MAC_TEMPLATE, RAW_MAC_TEMPLATE, LEGACY_MAC_TEMPLATE]
  )
  def test_verify_short_mac_fails(self, template):
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(mac.Mac)
    with self.assertRaises(tink.TinkError):
      primitive.verify_mac(b'', b'data')
    with self.assertRaises(tink.TinkError):
      primitive.verify_mac(b'tag', b'data')

  @parameterized.parameters([
      (MAC_TEMPLATE, MAC_TEMPLATE),
      (MAC_TEMPLATE, RAW_MAC_TEMPLATE),
      (MAC_TEMPLATE, LEGACY_MAC_TEMPLATE),
      (RAW_MAC_TEMPLATE, MAC_TEMPLATE),
      (RAW_MAC_TEMPLATE, RAW_MAC_TEMPLATE),
      (RAW_MAC_TEMPLATE, LEGACY_MAC_TEMPLATE),
      (LEGACY_MAC_TEMPLATE, MAC_TEMPLATE),
      (LEGACY_MAC_TEMPLATE, RAW_MAC_TEMPLATE),
      (LEGACY_MAC_TEMPLATE, LEGACY_MAC_TEMPLATE),
  ])
  def test_key_rotation(self, old_key_tmpl, new_key_tmpl):
    builder = keyset_builder.new_keyset_builder()
    older_key_id = builder.add_new_key(old_key_tmpl)

    builder.set_primary_key(older_key_id)
    mac1 = builder.keyset_handle().primitive(mac.Mac)

    newer_key_id = builder.add_new_key(new_key_tmpl)
    mac2 = builder.keyset_handle().primitive(mac.Mac)

    builder.set_primary_key(newer_key_id)
    mac3 = builder.keyset_handle().primitive(mac.Mac)

    builder.disable_key(older_key_id)
    mac4 = builder.keyset_handle().primitive(mac.Mac)

    self.assertNotEqual(older_key_id, newer_key_id)
    # 1 uses the older key. So 1, 2 and 3 can verify the mac, but not 4.
    mac_value1 = mac1.compute_mac(b'plaintext')
    mac1.verify_mac(mac_value1, b'plaintext')
    mac2.verify_mac(mac_value1, b'plaintext')
    mac3.verify_mac(mac_value1, b'plaintext')
    with self.assertRaises(tink.TinkError):
      mac4.verify_mac(mac_value1, b'plaintext')

    # 2 uses the older key. So 1, 2 and 3 can verify the mac, but not 4.
    mac_value2 = mac2.compute_mac(b'plaintext')
    mac1.verify_mac(mac_value2, b'plaintext')
    mac2.verify_mac(mac_value2, b'plaintext')
    mac3.verify_mac(mac_value2, b'plaintext')
    with self.assertRaises(tink.TinkError):
      mac4.verify_mac(mac_value2, b'plaintext')

    # 3 uses the newer key. So 2, 3 and 4 can verify the mac, but not 1.
    mac_value3 = mac3.compute_mac(b'plaintext')
    with self.assertRaises(tink.TinkError):
      mac1.verify_mac(mac_value3, b'plaintext')
    mac2.verify_mac(mac_value3, b'plaintext')
    mac3.verify_mac(mac_value3, b'plaintext')
    mac4.verify_mac(mac_value3, b'plaintext')

    # 4 uses the newer key. So 2, 3 and 4 can verify the mac, but not 1.
    mac_value4 = mac4.compute_mac(b'plaintext')
    with self.assertRaises(tink.TinkError):
      mac1.verify_mac(mac_value4, b'plaintext')
    mac2.verify_mac(mac_value4, b'plaintext')
    mac3.verify_mac(mac_value4, b'plaintext')
    mac4.verify_mac(mac_value4, b'plaintext')


class KeyUsageMonitorTest(absltest.TestCase):

  def setUp(self):
    super().setUp()
    self.compute_key_usage_monitor = (
        fake_key_usage_monitor.FakeKeyUsageMonitor()
    )
    self.verify_key_usage_monitor = fake_key_usage_monitor.FakeKeyUsageMonitor()
    _monitoring.register_key_usage_monitor_factory(
        lambda context: (
            self.compute_key_usage_monitor
            if context.get_api_function() == 'compute_mac'
            else self.verify_key_usage_monitor
        )
    )

  def test_key_usage_monitor_log(self):
    keyset_handle = _keyset_handle._new_keyset_handle_with_annotations(
        MAC_TEMPLATE, {'test': 'test'}
    )
    primitive = keyset_handle.primitive(mac.Mac)

    tag = primitive.compute_mac(b'data')
    primitive.verify_mac(tag, b'data')

    self.assertSequenceEqual(
        self.compute_key_usage_monitor.log_calls,
        [(keyset_handle.keyset_info().key_info[0].key_id, len(b'data'))],
    )
    self.assertSequenceEqual(
        self.verify_key_usage_monitor.log_calls,
        [(
            keyset_handle.keyset_info().key_info[0].key_id,
            len(tag) - core.crypto_format.NON_RAW_PREFIX_SIZE,
        )],
    )

  def test_key_usage_monitor_log_failure(self):
    keyset_handle = _keyset_handle._new_keyset_handle_with_annotations(
        MAC_TEMPLATE, {'test': 'test'}
    )
    primitive = keyset_handle.primitive(mac.Mac)

    try:
      primitive.verify_mac(b'x', b'data')
    except tink.TinkError:
      pass

    self.assertEqual(self.verify_key_usage_monitor.log_failure_calls_count, 1)


if __name__ == '__main__':
  absltest.main()



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A file-like object that encrypts data written to it.

It writes the ciphertext to a given other file-like object, which can later be
decrypted and read using a DecryptingStream wrapper.
"""

import io
from typing import BinaryIO, Optional

from tink import core
from tink.cc.pybind import tink_bindings
from tink.streaming_aead import _file_object_adapter


@core.use_tink_errors
def _new_cc_encrypting_stream(cc_primitive, aad, destination):
  """Implemented as a separate function to ensure correct error transform."""
  return tink_bindings.new_cc_encrypting_stream(
      cc_primitive, aad, destination)


class RawEncryptingStream(io.RawIOBase):
  """A file-like object which wraps writes to an underlying file-like object.

  It encrypts any data written to it, and writes the ciphertext to the wrapped
  object.

  The close() method indicates that the message is complete, and will write a
  final ciphertext block to signify end of message.
  """

  def __init__(self, stream_aead: tink_bindings.StreamingAead,
               ciphertext_destination: BinaryIO, associated_data: bytes):
    """Create a new RawEncryptingStream.

    Args:
      stream_aead: C++ StreamingAead primitive from which a C++ EncryptingStream
        will be obtained.
      ciphertext_destination: A writable file-like object to which ciphertext
        bytes will be written.
      associated_data: The associated data to use for encryption. This must
        match the associated_data used for decryption.
    """
    super().__init__()
    if not ciphertext_destination.writable():
      raise ValueError('ciphertext_destination must be writable')
    cc_ciphertext_destination = _file_object_adapter.FileObjectAdapter(
        ciphertext_destination)
    self._cc_encrypting_stream = _new_cc_encrypting_stream(
        stream_aead, associated_data, cc_ciphertext_destination)

  @core.use_tink_errors
  def _write_to_cc_encrypting_stream(self, b: bytes) -> int:
    return self._cc_encrypting_stream.write(bytes(b))

  @core.use_tink_errors
  def _close_cc_encrypting_stream(self) -> None:
    self._cc_encrypting_stream.close()

  def readinto(self, b: bytearray) -> Optional[int]:
    raise io.UnsupportedOperation()

  # b has type "Buffer", which is not yet supported by pytype.
  def write(self, b) -> Optional[int]:
    """Write the given buffer to the IO stream.

    Args:
      b: The buffer to write.
    Returns:
      The number of bytes written, which may be less than the length of b in
      bytes.
    Raises:
      TinkError: if there was a permanent error.

    """
    if self.closed:  # pylint:disable=using-constant-test
      raise ValueError('write on closed file')

    if not isinstance(b, (bytes, memoryview, bytearray)):
      raise TypeError('a bytes-like object is required, not {}'.format(
          type(b).__name__))
    written = self._write_to_cc_encrypting_stream(b)
    if written < 0 or written > len(b):
      raise core.TinkError('Incorrect number of bytes written')
    return written

  def close(self) -> None:
    """Flush and close the stream. Has no effect on a closed stream."""
    if self.closed:  # pylint:disable=using-constant-test
      return
    self.flush()
    self._close_cc_encrypting_stream()
    super().close()

  def writable(self) -> bool:
    """Return True if the stream supports writing."""
    return True



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Pre-generated KeyTemplate for Mac.

One can use these templates to generate a new tink_pb2.Keyset with
tink_pb2.KeysetHandle. To generate a new keyset that contains a single
hmac_pb2.HmacKey, one can do:
handle = keyset_handle.KeysetHandle(mac_key_templates.HMAC_SHA256_128BITTAG).
"""

import warnings

from tink.proto import aes_cmac_pb2
from tink.proto import common_pb2
from tink.proto import hmac_pb2
from tink.proto import tink_pb2


def _create_hmac_key_template(
    key_size: int, tag_size: int,
    hash_type: common_pb2.HashType) -> tink_pb2.KeyTemplate:
  """Creates a HMAC KeyTemplate, and fills in its values."""
  key_format = hmac_pb2.HmacKeyFormat()
  key_format.params.hash = hash_type
  key_format.params.tag_size = tag_size
  key_format.key_size = key_size
  key_template = tink_pb2.KeyTemplate(
      value=key_format.SerializeToString(),
      type_url='type.googleapis.com/google.crypto.tink.HmacKey',
      output_prefix_type=tink_pb2.TINK,
  )
  return key_template


def _create_aes_cmac_key_template(key_size: int,
                                  tag_size: int) -> tink_pb2.KeyTemplate:
  """"Creates an AES-CMAC KeyTemplate, and fills in its values."""
  key_format = aes_cmac_pb2.AesCmacKeyFormat()
  key_format.key_size = key_size
  key_format.params.tag_size = tag_size
  key_template = tink_pb2.KeyTemplate()
  key_template.value = key_format.SerializeToString()
  key_template.type_url = 'type.googleapis.com/google.crypto.tink.AesCmacKey'
  key_template.output_prefix_type = tink_pb2.TINK
  return key_template


AES_CMAC = _create_aes_cmac_key_template(key_size=32, tag_size=16)
HMAC_SHA256_128BITTAG = _create_hmac_key_template(
    key_size=32, tag_size=16, hash_type=common_pb2.SHA256)
HMAC_SHA256_256BITTAG = _create_hmac_key_template(
    key_size=32, tag_size=32, hash_type=common_pb2.SHA256)
HMAC_SHA512_256BITTAG = _create_hmac_key_template(
    key_size=64, tag_size=32, hash_type=common_pb2.SHA512)
HMAC_SHA512_512BITTAG = _create_hmac_key_template(
    key_size=64, tag_size=64, hash_type=common_pb2.SHA512)


# Deprecated. Use the predefined constant templates above instead.
def create_hmac_key_template(
    key_size: int, tag_size: int,
    hash_type: common_pb2.HashType) -> tink_pb2.KeyTemplate:
  warnings.warn('The "create_hmac_key_template" function is deprecated.',
                DeprecationWarning, 2)
  return _create_hmac_key_template(key_size, tag_size, hash_type)


# Deprecated. Use the predefined constant templates above instead.
def create_aes_cmac_key_template(key_size: int,
                                 tag_size: int) -> tink_pb2.KeyTemplate:
  warnings.warn('The "create_hmac_key_template" function is deprecated.',
                DeprecationWarning, 2)
  return _create_aes_cmac_key_template(key_size, tag_size)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interface for JwtPublicKeySign."""

import abc
from typing import Optional

from tink.jwt import _raw_jwt


class JwtPublicKeySign(metaclass=abc.ABCMeta):
  """Interface for creating a signed JWT.

  Sees RFC 7519 and RFC 7515. Security guarantees: similar to PublicKeySign.
  """

  @abc.abstractmethod
  def sign_and_encode(self, raw_jwt: _raw_jwt.RawJwt) -> str:
    """Computes a signature and encodes the token.

    Args:
      raw_jwt: The RawJwt token to be signed and encoded.

    Returns:
      The signed token encoded in the JWS compact serialization format.
    Raises:
      tink.TinkError if the operation fails.
    """
    raise NotImplementedError()


class JwtPublicKeySignInternal(metaclass=abc.ABCMeta):
  """Internal interface for creating a signed JWT.

  "kid" is an optional value that is set by the wrapper for keys with output
  prefix TINK, and it is set to None for output prefix RAW.
  """

  @abc.abstractmethod
  def sign_and_encode_with_kid(self, token: _raw_jwt.RawJwt,
                               kid: Optional[str]) -> str:
    raise NotImplementedError()



# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A container class for a set of primitives."""

import collections
from typing import Generic, List, Type, TypeVar, Optional, Dict

from tink.proto import tink_pb2
from tink.core import _crypto_format
from tink.core import _tink_error

P = TypeVar('P')
Entry = collections.namedtuple(
    'Entry', 'primitive, identifier, status, output_prefix_type, key_id')


def new_primitive_set(primitive_class):
  return PrimitiveSet(primitive_class)


class PrimitiveSet(Generic[P]):
  """A container class for a set of primitives.

  PrimitiveSet is an auxiliary class used for supporting key rotation:
  primitives in a set correspond to keys in a keyset. Users will usually work
  with primitive instances, which essentially wrap primitive sets. For example
  an instance of an Aead-primitive for a given keyset holds a set of
  Aead-primitives corresponding to the keys in the keyset, and uses the set
  members to do the actual crypto operations: to encrypt data the primary
  Aead-primitive from the set is used, and upon decryption the ciphertext's
  prefix determines the id of the primitive from the set.
  """

  def __init__(self, primitive_class: Type[P]):
    self._primitives: Dict[bytes, List[Entry]] = {}
    self._primary: Optional[Entry] = None
    self._primitive_class = primitive_class

  def primitive_class(self) -> Type[P]:
    return self._primitive_class

  def primitive_from_identifier(self, identifier: bytes) -> List[Entry]:
    """Returns a copy of the list of entries for a given identifier."""
    # Copy the list so that if the user modifies the list, it does not affect
    # the internal data structure.
    return self._primitives.get(identifier, [])[:]

  def all(self) -> List[List[Entry]]:
    """Returns a list of copies of all lists of entries in the primitive set."""
    return list(entries[:] for entries in self._primitives.values())

  def primitive(self, key: tink_pb2.Keyset.Key) -> List[Entry]:
    """Returns a copy of the list of entries for a given key."""
    return self.primitive_from_identifier(_crypto_format.output_prefix(key))

  def raw_primitives(self) -> List[Entry]:
    """Returns a copy of the list of entries of keys with raw prefix."""
    # All raw keys have the same identifier, which is just b''.
    return self.primitive_from_identifier(_crypto_format.RAW_PREFIX)

  def add_primitive(self, primitive: P, key: tink_pb2.Keyset.Key) -> Entry:
    """Adds a new primitive and key entry to the set, and returns the entry."""
    if not isinstance(primitive, self._primitive_class):
      raise _tink_error.TinkError(
          'The primitive is not an instance of {}'.format(
              self._primitive_class))
    identifier = _crypto_format.output_prefix(key)

    entry = Entry(primitive, identifier, key.status, key.output_prefix_type,
                  key.key_id)
    entries = self._primitives.setdefault(identifier, [])
    entries.append(entry)
    return entry

  def set_primary(self, entry: Entry) -> None:
    self._primary = entry

  def primary(self) -> Entry:
    if not self._primary:
      raise _tink_error.TinkError('The primary entry is not set.')
    return self._primary



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/ecies_aead_hkdf.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/ecies_aead_hkdf.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2
from tink.proto import tink_pb2 as tink_dot_proto_dot_tink__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tink/proto/ecies_aead_hkdf.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\x1a\x15tink/proto/tink.proto\"\x98\x01\n\x12\x45\x63iesHkdfKemParams\x12\x39\n\ncurve_type\x18\x01 \x01(\x0e\x32%.google.crypto.tink.EllipticCurveType\x12\x34\n\x0ehkdf_hash_type\x18\x02 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\x12\x11\n\thkdf_salt\x18\x0b \x01(\x0c\"G\n\x12\x45\x63iesAeadDemParams\x12\x31\n\x08\x61\x65\x61\x64_dem\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.KeyTemplate\"\xc9\x01\n\x13\x45\x63iesAeadHkdfParams\x12:\n\nkem_params\x18\x01 \x01(\x0b\x32&.google.crypto.tink.EciesHkdfKemParams\x12:\n\ndem_params\x18\x02 \x01(\x0b\x32&.google.crypto.tink.EciesAeadDemParams\x12:\n\x0f\x65\x63_point_format\x18\x03 \x01(\x0e\x32!.google.crypto.tink.EcPointFormat\"x\n\x16\x45\x63iesAeadHkdfPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x37\n\x06params\x18\x02 \x01(\x0b\x32\'.google.crypto.tink.EciesAeadHkdfParams\x12\t\n\x01x\x18\x03 \x01(\x0c\x12\t\n\x01y\x18\x04 \x01(\x0c\"}\n\x17\x45\x63iesAeadHkdfPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12>\n\npublic_key\x18\x02 \x01(\x0b\x32*.google.crypto.tink.EciesAeadHkdfPublicKey\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\"Q\n\x16\x45\x63iesAeadHkdfKeyFormat\x12\x37\n\x06params\x18\x01 \x01(\x0b\x32\'.google.crypto.tink.EciesAeadHkdfParamsBb\n\x1c\x63om.google.crypto.tink.protoP\x01Z@github.com/tink-crypto/tink-go/v2/proto/ecies_aead_hkdf_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.ecies_aead_hkdf_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z@github.com/tink-crypto/tink-go/v2/proto/ecies_aead_hkdf_go_proto'
  _globals['_ECIESHKDFKEMPARAMS']._serialized_start=105
  _globals['_ECIESHKDFKEMPARAMS']._serialized_end=257
  _globals['_ECIESAEADDEMPARAMS']._serialized_start=259
  _globals['_ECIESAEADDEMPARAMS']._serialized_end=330
  _globals['_ECIESAEADHKDFPARAMS']._serialized_start=333
  _globals['_ECIESAEADHKDFPARAMS']._serialized_end=534
  _globals['_ECIESAEADHKDFPUBLICKEY']._serialized_start=536
  _globals['_ECIESAEADHKDFPUBLICKEY']._serialized_end=656
  _globals['_ECIESAEADHKDFPRIVATEKEY']._serialized_start=658
  _globals['_ECIESAEADHKDFPRIVATEKEY']._serialized_end=783
  _globals['_ECIESAEADHKDFKEYFORMAT']._serialized_start=785
  _globals['_ECIESAEADHKDFKEYFORMAT']._serialized_end=866
# @@protoc_insertion_point(module_scope)



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Python KMS AEAD key manager."""


from typing import Type

from tink.proto import kms_aead_pb2
from tink.proto import kms_envelope_pb2
from tink.proto import tink_pb2
from tink import _kms_clients
from tink import core
from tink.aead import _aead
from tink.aead import _kms_envelope_aead


_KMS_AEAD_KEY_TYPE_URL = 'type.googleapis.com/google.crypto.tink.KmsAeadKey'
_KMS_ENVELOPE_AEAD_KEY_TYPE_URL = (
    'type.googleapis.com/google.crypto.tink.KmsEnvelopeAeadKey'
)


class KmsAeadKeyManager(core.KeyManager[_aead.Aead]):
  """KmsAeadKeyManager."""

  def primitive_class(self) -> Type[_aead.Aead]:
    return _aead.Aead

  def primitive(self, key_data: tink_pb2.KeyData) -> _aead.Aead:
    if key_data.type_url != _KMS_AEAD_KEY_TYPE_URL:
      raise core.TinkError('wrong key type: ' + key_data.type_url)
    kms_key = kms_aead_pb2.KmsAeadKey.FromString(key_data.value)
    client = _kms_clients.kms_client_from_uri(kms_key.params.key_uri)
    return client.get_aead(key_uri=kms_key.params.key_uri)

  def key_type(self) -> str:
    return _KMS_AEAD_KEY_TYPE_URL

  def new_key_data(
      self, key_template: tink_pb2.KeyTemplate
  ) -> tink_pb2.KeyData:
    if key_template.type_url != _KMS_AEAD_KEY_TYPE_URL:
      raise core.TinkError('wrong key type: ' + key_template.type_url)
    key = kms_aead_pb2.KmsAeadKey(
        version=0,
        params=kms_aead_pb2.KmsAeadKeyFormat.FromString(key_template.value),
    )
    return tink_pb2.KeyData(
        type_url=_KMS_AEAD_KEY_TYPE_URL,
        value=key.SerializeToString(),
        key_material_type=tink_pb2.KeyData.REMOTE,
    )

  def does_support(self, type_url: str) -> bool:
    return self.key_type() == type_url


class KmsEnvelopeAeadKeyManager(core.KeyManager[_aead.Aead]):
  """KmsEnvelopeAeadKeyManager."""

  def primitive_class(self) -> Type[_aead.Aead]:
    return _aead.Aead

  def primitive(self, key_data: tink_pb2.KeyData) -> _aead.Aead:
    if key_data.type_url != _KMS_ENVELOPE_AEAD_KEY_TYPE_URL:
      raise core.TinkError('wrong key type: ' + key_data.type_url)
    env_key = kms_envelope_pb2.KmsEnvelopeAeadKey.FromString(key_data.value)
    client = _kms_clients.kms_client_from_uri(env_key.params.kek_uri)

    return _kms_envelope_aead.KmsEnvelopeAead(
        env_key.params.dek_template,
        client.get_aead(key_uri=env_key.params.kek_uri),
    )

  def key_type(self) -> str:
    return _KMS_ENVELOPE_AEAD_KEY_TYPE_URL

  def new_key_data(
      self, key_template: tink_pb2.KeyTemplate
  ) -> tink_pb2.KeyData:
    if key_template.type_url != _KMS_ENVELOPE_AEAD_KEY_TYPE_URL:
      raise core.TinkError('wrong key type: ' + key_template.type_url)
    params = kms_envelope_pb2.KmsEnvelopeAeadKeyFormat.FromString(
        key_template.value
    )
    if not _kms_envelope_aead.is_supported_dek_key_type(
        params.dek_template.type_url
    ):
      raise core.TinkError(
          'Unsupported DEK key type: %s' % key_template.type_url
      )
    env_key = kms_envelope_pb2.KmsEnvelopeAeadKey(
        version=0,
        params=params,
    )
    return tink_pb2.KeyData(
        type_url=_KMS_ENVELOPE_AEAD_KEY_TYPE_URL,
        value=env_key.SerializeToString(),
        key_material_type=tink_pb2.KeyData.REMOTE,
    )

  def does_support(self, type_url: str) -> bool:
    return self.key_type() == type_url



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/aes_gcm_hkdf_streaming.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/aes_gcm_hkdf_streaming.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'tink/proto/aes_gcm_hkdf_streaming.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\"\x8c\x01\n\x19\x41\x65sGcmHkdfStreamingParams\x12\x1f\n\x17\x63iphertext_segment_size\x18\x01 \x01(\r\x12\x18\n\x10\x64\x65rived_key_size\x18\x02 \x01(\r\x12\x34\n\x0ehkdf_hash_type\x18\x03 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\"\x80\x01\n\x1c\x41\x65sGcmHkdfStreamingKeyFormat\x12\x0f\n\x07version\x18\x03 \x01(\r\x12=\n\x06params\x18\x01 \x01(\x0b\x32-.google.crypto.tink.AesGcmHkdfStreamingParams\x12\x10\n\x08key_size\x18\x02 \x01(\r\"{\n\x16\x41\x65sGcmHkdfStreamingKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12=\n\x06params\x18\x02 \x01(\x0b\x32-.google.crypto.tink.AesGcmHkdfStreamingParams\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42i\n\x1c\x63om.google.crypto.tink.protoP\x01ZGgithub.com/tink-crypto/tink-go/v2/proto/aes_gcm_hkdf_streaming_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.aes_gcm_hkdf_streaming_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZGgithub.com/tink-crypto/tink-go/v2/proto/aes_gcm_hkdf_streaming_go_proto'
  _globals['_AESGCMHKDFSTREAMINGPARAMS']._serialized_start=89
  _globals['_AESGCMHKDFSTREAMINGPARAMS']._serialized_end=229
  _globals['_AESGCMHKDFSTREAMINGKEYFORMAT']._serialized_start=232
  _globals['_AESGCMHKDFSTREAMINGKEYFORMAT']._serialized_end=360
  _globals['_AESGCMHKDFSTREAMINGKEY']._serialized_start=362
  _globals['_AESGCMHKDFSTREAMINGKEY']._serialized_end=485
# @@protoc_insertion_point(module_scope)



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tink.python.tink.testing._fake_kms."""

from absl.testing import absltest
import tink
from tink import aead
from tink.testing import fake_kms


KEY_URI = (
    'fake-kms://CL3oi0kSVwpMCjB0eXBlLmdvb2dsZWFwaXMuY29tL2dvb2dsZS5jcnlwdG8'
    'udGluay5BZXNFYXhLZXkSFhICCBAaEPFnQNgtxEG0vEek8bBfgL8YARABGL3oi0kgAQ')


def setUpModule():
  aead.register()
  fake_kms.register_client()


class FakeKmsTest(absltest.TestCase):

  def test_fake_kms_doesn_not_support_other_kms(self):
    client = fake_kms.FakeKmsClient()
    self.assertFalse(
        client.does_support('aws-kms://arn:aws:kms:us-east-2:12345:key/12345')
    )

  def test_fake_kms_aead_encrypt_decrypt(self):
    template = aead.aead_key_templates.create_kms_aead_key_template(
        key_uri=KEY_URI)
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = primitive.encrypt(plaintext, associated_data)
    self.assertEqual(primitive.decrypt(ciphertext, associated_data), plaintext)

  def test_fake_kms_envelope_encrypt_decrypt(self):
    template = aead.aead_key_templates.create_kms_envelope_aead_key_template(
        kek_uri=KEY_URI,
        dek_template=aead.aead_key_templates.AES128_GCM)
    keyset_handle = tink.new_keyset_handle(template)
    primitive = keyset_handle.primitive(aead.Aead)
    plaintext = b'plaintext'
    associated_data = b'associated_data'
    ciphertext = primitive.encrypt(plaintext, associated_data)
    self.assertEqual(primitive.decrypt(ciphertext, associated_data), plaintext)


if __name__ == '__main__':
  absltest.main()
